package com.huimv.guowei.admin.controller;


import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;

import com.huimv.guowei.admin.entity.DuckEggTextureInfo;
import com.huimv.guowei.admin.service.IBaseDuckInfoService;
import com.huimv.guowei.admin.service.IDuckEggTextureInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author author
 * @since 2023-06-14
 */
@RestController
@RequestMapping("/duck-egg-texture-info")
@CrossOrigin
public class DuckEggTextureInfoController {
    @Autowired
    private IDuckEggTextureInfoService duckEggTextureInfoService;
    @Autowired
    private IBaseDuckInfoService baseDuckInfoService;

    @PostMapping("/addInfo")
    public Result addInfo(@RequestBody DuckEggTextureInfo duckEggTextureInfo) throws ParseException {
        BaseDuckInfo baseDuckInfo = baseDuckInfoService.getOne(new QueryWrapper<BaseDuckInfo>().eq("duck_num", duckEggTextureInfo.getDuckNum()));
        if (baseDuckInfo.getDuckSex() == 0){
            return new Result(10001,"雄鸭无法新增蛋质信息!",false);
        }
        if (baseDuckInfo.getIsCage() == 1){
            return new Result(10001,"鸭只已离场!",false);
        }
        Date determineTime = duckEggTextureInfo.getDetermineTime();
        String layEggsTime = duckEggTextureInfo.getLayEggsTime();
        duckEggTextureInfo.setDuckId(baseDuckInfo.getId());
        duckEggTextureInfo.setChiNum(baseDuckInfo.getChiNum());
        duckEggTextureInfo.setJiaoNum(baseDuckInfo.getJiaoNum());
        duckEggTextureInfo.setBatchNum(baseDuckInfo.getBatchNum());
        String[] dateParts = layEggsTime.split("~");
        String startDateString = dateParts[0];
        String endDateString = dateParts[1];
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date startDate = sdf.parse(startDateString);
        Date endDate = sdf.parse(endDateString);

        if (startDate.compareTo(determineTime) > 0){
            return new Result(10001,"测定日期不能早于产蛋日期!",false);
        }
        duckEggTextureInfoService.save(duckEggTextureInfo);
        return Result.SUCCESS();
    }

    @PostMapping("/deleteInfo")
    public Result deleteInfo(@RequestBody Map<String, String> paramsMap){
        String id = paramsMap.get("id");
        duckEggTextureInfoService.removeById(id);
        return Result.SUCCESS();
    }

    @PostMapping("/updateInfo")
    public Result updateInfo(@RequestBody DuckEggTextureInfo duckEggTextureInfo){
        duckEggTextureInfoService.updateById(duckEggTextureInfo);
        return Result.SUCCESS();
    }

    @PostMapping("/listInfo")
    public Result listInfo(@RequestBody Map<String, String> paramsMap){
        return duckEggTextureInfoService.listLimitInfo(paramsMap);
    }

    @PostMapping("/getById")
    public Result getById(@RequestBody Map<String, String> paramsMap){
        return new Result(ResultCode.SUCCESS,duckEggTextureInfoService.getById(paramsMap.get("id")));
    }

    @PostMapping("/listPageInfo")
    public Result listPageInfo(@RequestBody Map<String, String> paramsMap){
        return duckEggTextureInfoService.listPageInfo(paramsMap);
    }

    //excel导入
    @PostMapping("/importData")
    public Result importData(HttpServletRequest httpServletRequest, @RequestParam(name = "file") MultipartFile file,
                             @RequestParam(name = "farmId") String farmId) throws Exception {
        Map<String, String> map = new HashMap<>();
        map.put("farmId", farmId);
        return duckEggTextureInfoService.importData(httpServletRequest, map, file);
    }
    //生成鸭蛋编号
    @PostMapping("/addEggsNum")
    public Result addEggsNum(@RequestBody Map<String, String> paramsMap){
        String farmId = paramsMap.get("farmId");
        QueryWrapper<DuckEggTextureInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("id").last(" limit 1");
        DuckEggTextureInfo one = duckEggTextureInfoService.getOne(queryWrapper);
        JSONObject jsonObject = new JSONObject();
        if (ObjectUtil.isEmpty(one)) {
            jsonObject.put("eggNum", "100001");
        } else {
            jsonObject.put("eggNum", Integer.parseInt(one.getEggsNum()) + 1);
        }
        return new Result(ResultCode.SUCCESS, jsonObject);
    }
}
