package com.huimv.guowei.admin.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.SysCamera;
import com.huimv.guowei.admin.entity.SysCameraArea;
import com.huimv.guowei.admin.service.ISysCameraAreaService;
import com.huimv.guowei.admin.service.ISysCameraService;
import com.huimv.guowei.admin.utils.IpUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@RestController
@RequestMapping("/admin/cameraArea")
@CrossOrigin
public class SysCameraAreaController {
    @Autowired
    private ISysCameraAreaService cameraAreaService;

    @Autowired
    private ISysCameraService cameraService;


    @PostMapping("/list")
    public Result list(@RequestBody Map<String, Object> params) {
        return cameraAreaService.cameraList(params);
    }

    @PostMapping("/add")
    public Result add(@RequestBody SysCameraArea cameraArea) {
        int brandName = cameraAreaService.count(new QueryWrapper<SysCameraArea>().eq("area_name", cameraArea.getAreaName()).eq("farm_id", cameraArea.getFarmId()));
        if (brandName > 0) {
            return new Result(100001, "区域名称重复", false);
        }
        cameraAreaService.save(cameraArea);
        return new Result(10000, "添加成功", true);
    }

    @PostMapping("/update")
    public Result update(@RequestBody SysCameraArea cameraArea) {
        cameraAreaService.updateById(cameraArea);
        int brandName = cameraAreaService.count(new QueryWrapper<SysCameraArea>().eq("area_name", cameraArea.getAreaName()).eq("farm_id",cameraArea.getFarmId()));
        if (brandName > 1) {
            return new Result(100001, "区域名称重复", false);
        }

        String areaName = cameraArea.getAreaName();
        SysCamera camera = new SysCamera();
        if (StringUtils.isNotBlank(areaName)) {
            camera.setAreaName(areaName);
        }
        UpdateWrapper<SysCamera> wrapper = new UpdateWrapper<>();
        wrapper.eq("area_id", cameraArea.getId());
        cameraService.update(camera, wrapper);


        return new Result(10000, "修改成功", true);
    }

    @PostMapping("/updateStatus")
    public Result updateStatus(@RequestBody Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        SysCameraArea sysCameraArea = cameraAreaService.getById(id);
        sysCameraArea.setRunStatus(Math.abs(1-sysCameraArea.getRunStatus()));
        cameraAreaService.updateById(sysCameraArea);
        return new Result(10000, "修改成功", true);
    }

    @GetMapping("/remove")
    public Result remove(@RequestParam(value = "ids") List<Integer> ids, @RequestParam(value = "farmId") String farmId) {

        return cameraAreaService.removeIds(ids,farmId);

    }


    @GetMapping("/lsitAndCamera")
    public Result lsitAndCamera(@RequestParam(value = "farmId") Integer farmId) {
        return cameraAreaService.lsitAndCamera(farmId);

    }

    @PostMapping("/listCameraByArea")
    public Result listCameraByArea(@RequestBody Map<String, String> paramsMap) {
        String areaId = paramsMap.get("areaId");
        List<SysCamera> list = cameraService.list(new QueryWrapper<SysCamera>().lambda().eq(SysCamera::getAreaId, areaId).eq(SysCamera::getRunStatus, 1).orderByAsc(SysCamera::getSort));
        return new Result(ResultCode.SUCCESS,list);

    }

    @GetMapping("/listScreenCamera")
    public Result listScreenCamera(@RequestParam(value = "farmId") Integer farmId) {
        return cameraAreaService.listScreenCamera(farmId);

    }


    @GetMapping("/getUrl")
    public void getUrl(HttpServletRequest request, @RequestParam(value = "farmId") Integer farmId) {
        String ipAddr = IpUtil.getIpAddr(request);
        QueryWrapper<SysCameraArea> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<SysCameraArea> areas = cameraAreaService.list(queryWrapper);
        for (SysCameraArea area : areas) {
            area.setPublicUrl(ipAddr+":9080");
            cameraAreaService.updateById(area);
        }
    }
}
