package com.huimv.guowei.admin.group.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.guowei.admin.common.utils.PdfUtil;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupDuckInfo;
import com.huimv.guowei.admin.group.service.IGroupDuckInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 平养鸭只档案 前端控制器
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@RestController
@RequestMapping("/group-duck-info")
@CrossOrigin
public class GroupDuckInfoController {

    @Autowired
    private IGroupDuckInfoService groupDuckInfoService;

    @RequestMapping("/listPage")
    public Result listPage(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.listPage(paramsMap);
    }

    @RequestMapping("/add")
    public Result addDuck(HttpServletRequest httpServletRequest, @RequestParam(name = "imgUrl", required = false) MultipartFile imgUrl,
                          @RequestParam(name = "farmId") String farmId,
                          @RequestParam(name = "deviceCode") String deviceCode,
                          @RequestParam(name = "chiCode", required = false) String chiCode,
                          @RequestParam(name = "batchNum") String batchNum,
                          @RequestParam(name = "duckBreed") String duckBreed,
                          @RequestParam(name = "sex") String sex,
                          @RequestParam(name = "birthDate") String birthDate,
                          @RequestParam(name = "weight") String weight,
                          @RequestParam(name = "unitId") String unitId,
                          @RequestParam(name = "unitName") String unitName) throws IOException, ParseException {

        httpServletRequest.setCharacterEncoding("UTF-8");
        Map<String, String> map = new HashMap<>();
        map.put("farmId", farmId);
        map.put("deviceCode", deviceCode);
        map.put("chiCode", chiCode);
        map.put("batchNum", batchNum);
        map.put("sex", sex);
        map.put("duckBreed", duckBreed);
        map.put("birthDate", birthDate);
        map.put("weight", weight);
        map.put("unitId", unitId);
        map.put("unitName", unitName);
        return groupDuckInfoService.addDuck(httpServletRequest, map, imgUrl);
    }

    @RequestMapping("/editDuck")
    public Result editDuck(HttpServletRequest httpServletRequest,@RequestBody GroupDuckInfo groupDuckInfo) {
        return groupDuckInfoService.editDuck(httpServletRequest,groupDuckInfo);
    }

    @RequestMapping("/remove")
    public Result deleteDuck(HttpServletRequest httpServletRequest,@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.deleteDuck(httpServletRequest,paramsMap);
    }

    @RequestMapping("/listDetail")
    public Result listDetail(HttpServletRequest httpServletRequest,@RequestBody Map<String, String> paramsMap) {
        return new Result(ResultCode.SUCCESS,groupDuckInfoService.getById(paramsMap.get("id")));
    }

    @RequestMapping("/list")
    public Result list(HttpServletRequest httpServletRequest,@RequestBody Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String keyword = paramsMap.get("keyword");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<GroupDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank(unitId), "unit_id", unitId);
        if (StringUtils.isNotBlank(keyword)) {
            queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                    .or().like(StringUtils.isNotBlank(keyword), "chi_code", keyword));
        }
        return new Result(ResultCode.SUCCESS,groupDuckInfoService.list(queryWrapper));
    }

    //后台平养鸭只档案信息带总采食量等等
    @RequestMapping("/duckDetail")
    public Result duckDetail(HttpServletRequest httpServletRequest,@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.duckDetail(httpServletRequest,paramsMap);
    }

    @RequestMapping("/breedingList")
    public Result breedingList(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.breedingList(paramsMap);
    }

    @RequestMapping("/breedingDetail")
    public Result breedingDetail(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.breedingDetail(paramsMap);
    }

    //导入档案数据
    @PostMapping("/importData")
    public Result importData(@RequestParam(name = "file") MultipartFile file
                             ) throws Exception {
        return groupDuckInfoService.importData(file);
    }
    //导出档案数据
    @GetMapping("/printDuckInfo")
    public void printChi(HttpServletResponse response, @RequestParam(name = "farmId") String farmId,
                         @RequestParam(name = "ids") String ids) throws Exception {
        Map<String, String> map = new HashMap<>();
        map.put("farmId", farmId);
        map.put("ids", ids);
        groupDuckInfoService.printDuckIfo(response, map);
        String path = "/opt/guowei/groupDuckInfo.xls";
        PdfUtil.returnPdfStream3(response, path, "平养档案");
    }

    //大屏平养区--生产数据
    @RequestMapping("/screenProd")
    public Result screenProd(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.screenProd(paramsMap);
    }

    //智慧选育大屏梯队排名
    @RequestMapping("/listRank")
    public Result listRank(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.listRank(paramsMap);
    }

    //智慧选育大屏点击梯队鸭只展示
    @RequestMapping("/listRankList")
    public Result listRankList(@RequestBody Map<String, String> paramsMap) {
        return groupDuckInfoService.listRankList(paramsMap);
    }
}
