package com.huimv.guowei.admin.group.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnergyWater;
import com.huimv.guowei.admin.group.entity.*;
import com.huimv.guowei.admin.group.mapper.*;
import com.huimv.guowei.admin.group.service.IGroupBatchService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.mapper.EnergyWaterMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平养批次 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupBatchServiceImpl extends ServiceImpl<GroupBatchMapper, GroupBatch> implements IGroupBatchService {

    @Autowired
    private GroupBatchMapper batchMapper;
    @Autowired
    private GroupDuckInfoMapper infoMapper;
    @Autowired
    private GroupEatingMapper eatingMapper;
    @Autowired
    private EnergyWaterMapper waterMapper;
    @Autowired
    private GroupEggMapper eggMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String batchNum = paramsMap.get("batchNum");
        Page<GroupBatch> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(batchNum), "batch_num", batchNum);
        return new Result(ResultCode.SUCCESS, batchMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(GroupBatch groupBatch) {
        Integer count = batchMapper.selectCount(new QueryWrapper<GroupBatch>().eq("batch_num", groupBatch.getBatchNum())
                .eq("farm_id", groupBatch.getFarmId()));
        if (count != 0) {
            return new Result(10001, "添加失败，批次号已存在", false);
        }
        batchMapper.insert(groupBatch);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result edit(GroupBatch groupBatch) {
        Integer count = batchMapper.selectCount(new QueryWrapper<GroupBatch>().eq("batch_num", groupBatch.getBatchNum())
                .eq("farm_id", groupBatch.getFarmId()).ne("id",groupBatch.getId()));
        if (count != 0) {
            return new Result(10001, "修改失败，批次号已存在", false);
        }
        String duckId = groupBatch.getDuckId();
        String[] split = duckId.split(",");
        for (String muskId : split) {
            GroupDuckInfo duckInfo = infoMapper.selectById(muskId);
            duckInfo.setBatchNum(groupBatch.getBatchNum());
            infoMapper.updateById(duckInfo);
        }
        batchMapper.updateById(groupBatch);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        GroupBatch groupBatch = batchMapper.selectById(id);
        String duckId = groupBatch.getDuckId();
        if (StringUtils.isNotBlank(duckId)) {
            String[] split = duckId.split(",");
            for (String muskId : split) {
                GroupDuckInfo duckInfo = infoMapper.selectById(muskId);
                duckInfo.setBatchNum(null);
                infoMapper.updateById(duckInfo);
            }
        }
        batchMapper.deleteById(id);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result screenBatch(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        List<GroupBatch> batches = batchMapper.selectList(new QueryWrapper<GroupBatch>().eq("farm_id", farmId));
        JSONArray jsonArray = new JSONArray();
        if (batches.size() != 0) {
            for (GroupBatch batch : batches) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("batch", batch.getBatchNum());
                QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", farmId).in("device_code", batch.getDuckId()).ge("eating_date", timesmorning);
                queryWrapper.select("IFNULL(sum(eating_value),'0') 'eatingValue'");
                GroupEating groupEating = eatingMapper.selectOne(queryWrapper);
                jsonObject.put("eating", groupEating.getEatingValue());
                QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("farm_id", farmId).eq("chip_code", 1006).ge("create_date", timesmorning);
                queryWrapper1.select("IFNULL(sum(water_value),'0') 'waterValue'");
                EnergyWater water = waterMapper.selectOne(queryWrapper1);
                jsonObject.put("water", water.getWaterValue());
                QueryWrapper<GroupEgg> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("farm_id", farmId).in("device_code", batch.getDuckId()).ge("egg_date", timesmorning);
                queryWrapper2.select("IFNULL(sum(egg_num),'0') 'eggNum'");
                GroupEgg egg = eggMapper.selectOne(queryWrapper2);
                jsonObject.put("egg", egg.getEggNum());
                queryWrapper2.clear();
                queryWrapper2.eq("farm_id", farmId).in("device_code", batch.getDuckId());
                queryWrapper2.select("IFNULL(sum(egg_num),'0') 'eggNum'");
                GroupEgg groupEgg = eggMapper.selectOne(queryWrapper2);
                jsonObject.put("eggCount", groupEgg.getEggNum());
                jsonObject.put("result", "健康");
                jsonArray.add(jsonObject);
            }
        }

        return new Result(ResultCode.SUCCESS, jsonArray);
    }
}
