package com.huimv.guowei.admin.group.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupEgg;
import com.huimv.guowei.admin.group.mapper.GroupEatingMapper;
import com.huimv.guowei.admin.group.service.IGroupEatingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 平养采食记录 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupEatingServiceImpl extends ServiceImpl<GroupEatingMapper, GroupEating> implements IGroupEatingService {

    @Autowired
    private GroupEatingMapper eatingMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String keyword = paramsMap.get("keyword");
        String time = paramsMap.get("time");
        Page<GroupEating> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("eating_date");
        if (StringUtils.isNotBlank(keyword)) {
            queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                    .or().like(StringUtils.isNotBlank(keyword), "batch_num", keyword));
        }
        queryWrapper.between(StringUtils.isNotBlank(time), "eating_date", time + " 00:00:00", time + " 23:59:59");
        return new Result(ResultCode.SUCCESS, eatingMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("id");
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("device_code", deviceCode);
        return new Result(ResultCode.SUCCESS, eatingMapper.listDetail(queryWrapper));
    }

    @Override
    public Result BreedingEating(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("deviceCode");
        String type = paramsMap.get("type");
        if ("".equals(type) || null == type) {
            type = "2";
        }
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("device_code", deviceCode);
        if ("1".equals(type)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("eating_date", timesmorning);
            return new Result(ResultCode.SUCCESS, eatingMapper.selectList(queryWrapper));
        } else if ("2".equals(type)) {
            DateTime beginOfWeek = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("eating_date", beginOfWeek);
            return new Result(ResultCode.SUCCESS, eatingMapper.breedList(queryWrapper));
        } else if ("3".equals(type)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("eating_date", monthmorning);
            return new Result(ResultCode.SUCCESS, eatingMapper.breedList(queryWrapper));
        } else {
            String startTime = paramsMap.get("startTime");
            String endTime = paramsMap.get("endTime");
            queryWrapper.between("eating_date", startTime, endTime);
            return new Result(ResultCode.SUCCESS, eatingMapper.selectList(queryWrapper));
        }
    }

    @Override
    public Result BreedingEatingEgg(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("deviceCode");
        String type = paramsMap.get("type");
        if ("".equals(type) || null == type) {
            type = "1";
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if ("1".equals(type)) {
            DateTime beginOfWeek = DateUtil.beginOfWeek(new Date());
            return new Result(ResultCode.SUCCESS, eatingMapper.breedEggEating(farmId, deviceCode, sdf.format(beginOfWeek)));
        } else if ("2".equals(type)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            return new Result(ResultCode.SUCCESS, eatingMapper.breedEggEating(farmId, deviceCode, sdf.format(monthmorning)));
        } else if ("3".equals(type)) {
            Date monthmorning = DateUtil.beginOfYear(new Date());
            return new Result(ResultCode.SUCCESS, eatingMapper.breedEggEating2(farmId, deviceCode, sdf.format(monthmorning)));
        } else {
            String startTime = paramsMap.get("startTime");
            String endTime = paramsMap.get("endTime");
            return new Result(ResultCode.SUCCESS, eatingMapper.breedEggEating1(farmId, deviceCode, startTime, endTime));
        }
    }

}
