package com.huimv.guowei.admin.group.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupEgg;
import com.huimv.guowei.admin.group.entity.vo.GroupEatingVo;
import com.huimv.guowei.admin.group.mapper.GroupEggMapper;
import com.huimv.guowei.admin.group.service.IGroupEggService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平养产蛋 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupEggServiceImpl extends ServiceImpl<GroupEggMapper, GroupEgg> implements IGroupEggService {

    @Autowired
    private GroupEggMapper eggMapper;

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String deviceCode = paramsMap.get("deviceCode");
        String farmId = paramsMap.get("farmId");
        QueryWrapper<GroupEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank(deviceCode), "device_code", deviceCode);
        return new Result(ResultCode.SUCCESS, eggMapper.listDetail(queryWrapper));
    }

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String keyword = paramsMap.get("keyword");
        String time = paramsMap.get("time");
        Page<GroupEgg> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("egg_date");
        if (StringUtils.isNotBlank(keyword)) {
            queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                    .or().like(StringUtils.isNotBlank(keyword), "batch_num", keyword));
        }
        queryWrapper.between(StringUtils.isNotBlank(time), "egg_date", time + " 00:00:00", time + " 23:59:59");
        return new Result(ResultCode.SUCCESS, eggMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result BreedingEgg(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("deviceCode");
        String type = paramsMap.get("type");
        if ("".equals(type) || null == type) {
            type = "2";
        }
        QueryWrapper<GroupEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("device_code", deviceCode);
        if ("1".equals(type)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("egg_date", timesmorning);
            return new Result(ResultCode.SUCCESS, eggMapper.selectList(queryWrapper));
        } else if ("2".equals(type)) {
            DateTime beginOfWeek = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("egg_date", beginOfWeek);
            return new Result(ResultCode.SUCCESS, eggMapper.listDetail1(queryWrapper));
        } else if ("3".equals(type)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("egg_date", monthmorning);
            return new Result(ResultCode.SUCCESS, eggMapper.listDetail1(queryWrapper));
        } else {
            String startTime = paramsMap.get("startTime");
            String endTime = paramsMap.get("endTime");
            queryWrapper.between("egg_date", startTime, endTime);
            return new Result(ResultCode.SUCCESS, eggMapper.selectList(queryWrapper));
        }
    }
}
