package com.huimv.guowei.admin.group.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.BaseDuckBreedImg;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.Batch;
import com.huimv.guowei.admin.entity.vo.BaseDuckInfoImportData;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupFeedingEgg;
import com.huimv.guowei.admin.group.entity.dto.ScreenBreedVo;
import com.huimv.guowei.admin.group.entity.vo.GroupFeedAndEggVo;
import com.huimv.guowei.admin.group.entity.vo.ImportEggVo;
import com.huimv.guowei.admin.group.entity.vo.ImportFeedingVo;
import com.huimv.guowei.admin.group.mapper.GroupFeedingEggMapper;
import com.huimv.guowei.admin.group.service.IGroupFeedingEggService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.mapper.BaseBuildingMapper;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import org.apache.commons.lang.time.DateUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平养投料捡蛋 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupFeedingEggServiceImpl extends ServiceImpl<GroupFeedingEggMapper, GroupFeedingEgg> implements IGroupFeedingEggService {

    @Autowired
    private GroupFeedingEggMapper feedingEggMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String unitId = paramsMap.get("unitId");
        String time = paramsMap.get("time");
        Page<GroupFeedingEgg> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupFeedingEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("type", type).orderByDesc("date_time")
                .eq(StringUtils.isNotBlank(unitId),"unit_id", unitId);
        queryWrapper.between(StringUtils.isNotBlank(time), "date_time", time + " 00:00:00", time + " 23:59:59");
        return new Result(ResultCode.SUCCESS, feedingEggMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");
        QueryWrapper<GroupFeedingEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id",farmId).eq("type",type).eq("id",id);
        return new Result(ResultCode.SUCCESS, feedingEggMapper.selectOne(queryWrapper));
    }

    @Override
    public Result add(GroupFeedingEgg feedingEgg) {
        if (feedingEgg.getType() == 1) {
            double v = Double.valueOf(feedingEgg.getEggWeight())*1000 / Double.valueOf(feedingEgg.getEggAvg());
            int num = (int) Math.round(v);
            feedingEgg.setEggNum(num);
        }
        feedingEgg.setCreateTime(new Date());
        feedingEggMapper.insert(feedingEgg);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result edit(GroupFeedingEgg feedingEgg) {
        if (feedingEgg.getType() == 1) {
            double v = Double.valueOf(feedingEgg.getEggWeight())*1000 / Double.valueOf(feedingEgg.getEggAvg());
            int num = (int) Math.round(v);
            feedingEgg.setEggNum(num);
        }
        feedingEggMapper.updateById(feedingEgg);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        feedingEggMapper.deleteById(id);
        return new Result(ResultCode.SUCCESS);
    }

    @Transactional
    @Override
    public Result importData(Map<String, String> paramsMap, MultipartFile file) {

        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("栋舍名称", "unitName");
        map.put("投料日期", "dateTime");
        map.put("饲料种类", "feedType");
        map.put("投料重量", "feedValue");

        try (
                //这里面的对象会自动关闭
                InputStream in = file.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, file.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<ImportFeedingVo> list = ExcelImportSheet.getListFromExcel(sheet, ImportFeedingVo.class, map);
            for (ImportFeedingVo feedingVo : list) {
                GroupFeedingEgg feedingEgg = new GroupFeedingEgg();
                QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("build_name", feedingVo.getUnitName()).eq("type", 1);
                BaseBuilding baseBuilding = buildingMapper.selectOne(queryWrapper);
                if (ObjectUtil.isEmpty(baseBuilding)) {
                    return new Result(10001, "栋舍名称不存在！", false);
                } else {
                    feedingEgg.setUnitName(feedingVo.getUnitName());
                    feedingEgg.setUnitId(baseBuilding.getId());
                    feedingEgg.setFarmId(baseBuilding.getFarmId());
                    feedingEgg.setType(0);
                    feedingEgg.setDateTime(feedingVo.getDateTime());
                    feedingEgg.setFeedValue(feedingVo.getFeedValue());
                    feedingEgg.setCreateTime(new Date());
                    feedingEgg.setFeedType(feedingVo.getFeedType());
                    feedingEggMapper.insert(feedingEgg);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        } finally {
            //写着好看的
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result importData1(Map<String, String> paramsMap, MultipartFile file) {
        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("栋舍名称", "unitName");
        map.put("捡蛋日期", "dateTime");
        map.put("捡蛋总重量(kg)", "eggWeight");
        map.put("捡蛋均重(g)", "eggAvg");

        try (
                //这里面的对象会自动关闭
                InputStream in = file.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, file.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<ImportEggVo> list = ExcelImportSheet.getListFromExcel(sheet, ImportEggVo.class, map);
            for (ImportEggVo feedingVo : list) {
                GroupFeedingEgg feedingEgg = new GroupFeedingEgg();
                QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("build_name", feedingVo.getUnitName()).eq("type", 1);
                BaseBuilding baseBuilding = buildingMapper.selectOne(queryWrapper);
                if (ObjectUtil.isEmpty(baseBuilding)) {
                    return new Result(10001, "栋舍名称不存在！", false);
                } else {
                    feedingEgg.setUnitName(feedingVo.getUnitName());
                    feedingEgg.setUnitId(baseBuilding.getId());
                    feedingEgg.setFarmId(baseBuilding.getFarmId());
                    feedingEgg.setType(1);
                    feedingEgg.setDateTime(feedingVo.getDateTime());
                    feedingEgg.setEggWeight(feedingVo.getEggWeight());
                    feedingEgg.setEggAvg(feedingVo.getEggAvg());
                    double v = Double.valueOf(feedingEgg.getEggWeight())*1000 / Double.valueOf(feedingEgg.getEggAvg());
                    int num = (int) Math.round(v);
                    feedingEgg.setEggNum(num);
                    feedingEgg.setCreateTime(new Date());
                    feedingEggMapper.insert(feedingEgg);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        } finally {
            //写着好看的
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result screenBreed(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        Date monthmorning = DataUill.getTimesMonthmorning();
        DateTime dateTime = DateUtil.beginOfYear(new Date());

        QueryWrapper<GroupFeedingEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id",farmId).eq("type",0).eq("unit_id",unitId);
        queryWrapper.ge("date_time", monthmorning);
        List<ScreenBreedVo> breedVos = feedingEggMapper.screenBreed(queryWrapper);
        queryWrapper.clear();
        queryWrapper.eq("farm_id",farmId).eq("type",0).eq("unit_id",unitId);
        queryWrapper.ge("date_time", dateTime);
        List<ScreenBreedVo> breedVos1 = feedingEggMapper.screenBreedMonth(queryWrapper);

        JSONObject jsonObject = new JSONObject();
        DecimalFormat sdf = new DecimalFormat("0.00");

        jsonObject.put("todayList", breedVos);
        if (breedVos.size() != 0) {
            jsonObject.put("todayFeeding", breedVos.get(breedVos.size() - 1).getValues());
            String feeding = breedVos.get(breedVos.size() - 1).getValues();
            if (breedVos.size()!=1) {
                String lastValues = breedVos.get(breedVos.size() - 2).getValues();
                double value = ((Double.valueOf(feeding) - Double.valueOf(lastValues)) / Double.valueOf(lastValues))*100;
                jsonObject.put("today", sdf.format(value));
            } else {
                jsonObject.put("today", 100);
            }
        } else {
            jsonObject.put("today", 0);
        }

        jsonObject.put("monthList", breedVos1);
        if (breedVos1.size() != 0) {
            jsonObject.put("monthFeeding", breedVos1.get(breedVos1.size() - 1).getValues());
            String feeding = breedVos1.get(breedVos1.size() - 1).getValues();
            if (breedVos1.size() != 1) {
                String lastValues = breedVos1.get(breedVos1.size() - 2).getValues();
                double value = ((Double.valueOf(feeding) - Double.valueOf(lastValues)) / Double.valueOf(lastValues))*100;
                jsonObject.put("month", sdf.format(value));
            } else {
                jsonObject.put("month", 100);
            }
            }
           else {
            jsonObject.put("month", 0);
        }
        return new Result(ResultCode.SUCCESS,jsonObject);
    }

}
