package com.huimv.guowei.admin.group.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupBatch;
import com.huimv.guowei.admin.group.entity.GroupDuckInfo;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupLeave;
import com.huimv.guowei.admin.group.mapper.GroupBatchMapper;
import com.huimv.guowei.admin.group.mapper.GroupDuckInfoMapper;
import com.huimv.guowei.admin.group.mapper.GroupLeaveMapper;
import com.huimv.guowei.admin.group.service.IGroupLeaveService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 平养离场记录 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupLeaveServiceImpl extends ServiceImpl<GroupLeaveMapper, GroupLeave> implements IGroupLeaveService {

    @Autowired
    private GroupLeaveMapper leaveMapper;
    @Autowired
    private GroupBatchMapper batchMapper;
    @Autowired
    private GroupDuckInfoMapper duckInfoMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String keyword = paramsMap.get("keyword");
        String time = paramsMap.get("time");
        Page<GroupLeave> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupLeave> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("leave_date");
        if (StringUtils.isNotBlank(keyword)) {
            queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                    .or().like(StringUtils.isNotBlank(keyword), "batch_num", keyword));
        }
        queryWrapper.between(StringUtils.isNotBlank(time), "leave_date", time + " 00:00:00", time + " 23:59:59");
        return new Result(ResultCode.SUCCESS, leaveMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(GroupLeave leave) {
        leave.setCreateTime(new Date());
        if (leave.getType() == 0) {
            String deviceCode = leave.getDeviceCode();
            QueryWrapper<GroupDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", leave.getFarmId()).eq("device_code", deviceCode);
            GroupDuckInfo duckInfo = duckInfoMapper.selectOne(queryWrapper);
            duckInfo.setIsLeave(1);
            duckInfoMapper.updateById(duckInfo);
        } else {
            String deviceCode = leave.getDeviceCode();
            QueryWrapper<GroupBatch> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", leave.getFarmId()).eq("batch_num", deviceCode);
            GroupBatch groupBatch = batchMapper.selectOne(queryWrapper);
            String duckId = groupBatch.getDuckId();
            String[] split = duckId.split(",");
            for (String muskId : split) {
                GroupDuckInfo duckInfo = duckInfoMapper.selectOne(new QueryWrapper<GroupDuckInfo>().eq("device_code", muskId));
                duckInfo.setIsLeave(1);
                duckInfoMapper.updateById(duckInfo);
            }
        }
        return new Result(ResultCode.SUCCESS, leaveMapper.insert(leave));
    }

    @Override
    public Result edit(GroupLeave leave) {
        if (leave.getIsWithdraw() == 1) {
            if (leave.getType() == 0) {
                String deviceCode = leave.getDeviceCode();
                QueryWrapper<GroupDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", leave.getFarmId()).eq("device_code", deviceCode);
                GroupDuckInfo duckInfo = duckInfoMapper.selectOne(queryWrapper);
                duckInfo.setIsLeave(0);
                duckInfoMapper.updateById(duckInfo);
            }else {
                String deviceCode = leave.getDeviceCode();
                QueryWrapper<GroupBatch> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", leave.getFarmId()).eq("batch_num", deviceCode);
                GroupBatch groupBatch = batchMapper.selectOne(queryWrapper);
                String duckId = groupBatch.getDuckId();
                String[] split = duckId.split(",");
                for (String muskId : split) {
                    GroupDuckInfo duckInfo = duckInfoMapper.selectOne(new QueryWrapper<GroupDuckInfo>().eq("device_code", muskId));
                    duckInfo.setIsLeave(0);
                    duckInfoMapper.updateById(duckInfo);
                }
            }
            leaveMapper.deleteById(leave.getId());
        }
        leaveMapper.updateById(leave);
        return new Result(ResultCode.SUCCESS);
    }
}
