package com.huimv.guowei.admin.mqtt.EnvControll;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.service.*;
import com.huimv.guowei.admin.utils.SpringUtil;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZonedDateTime;
import java.util.*;

@Component
//接收消息回调
@Slf4j
public class PushCallback implements MqttCallbackExtended {

    @Override
    public void connectComplete(boolean b, String s) {
        System.out.println("重新订阅！");

    }

    @Override
    public void connectionLost(Throwable cause) {
        // 连接丢失后，一般在这里面进行重连
        System.out.println("连接断开，可以做重连");

    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        System.out.println("deliveryComplete---------" + token.isComplete());
    }

    @Override
    public void messageArrived(String topic, MqttMessage message) throws Exception {
        // subscribe后得到的消息会执行到这里面
        System.out.println("接收消息主题 : " + topic);
        System.out.println("接收消息Qos : " + message.getQos());
        System.out.println("接收消息内容 : " + new String(message.getPayload()));
        System.out.println("开始处理当前数据...");

        String[] split = topic.split("_");
        String chip = split[2];
        System.out.println(chip);
        String messages = new String(message.getPayload());

        if ("sync_time".equals(messages)) {
            log.info("-----开始配置同步时间----");
            JSONObject jsonObject1 = new JSONObject();
            ZonedDateTime ZonedDateTime = java.time.ZonedDateTime.now();
            Map<String, Object> map = new HashMap<>();
            map.put("year", ZonedDateTime.getYear());
            map.put("mon", ZonedDateTime.getMonthValue());
            map.put("date", ZonedDateTime.getDayOfMonth());
            map.put("hour", ZonedDateTime.getHour());
            map.put("min", ZonedDateTime.getMinute());
            map.put("sec", ZonedDateTime.getSecond());
            jsonObject1.put("sync_time", map);
            System.out.println(jsonObject1);
            try {
                PublishMQTT publishMQTT = new PublishMQTT();
                publishMQTT.start("tcp://115.238.57.190:1883", "GUOWEIpublish", "admin", "admin", "huimv_down_" + chip, jsonObject1.toJSONString());
            } catch (Exception e) {
                System.out.println(e);
            }
        } else if ("alarm_cfg".equals(messages)) {
            log.info("高低温报警不做处理！！");
        } else if ("get_dev_ctrl".equals(messages)) {
            JSONObject jsonObject = new JSONObject();
            log.info("-----开始配置设备控制消息----");
            QueryWrapper<EnvDevice> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_code", chip);
            IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
            EnvDevice one = envDeviceService.getOne(queryWrapper1);

            IEnvDeviceEquipmentService envDeviceEquipmentService = SpringUtil.getBean(IEnvDeviceEquipmentService.class);
            QueryWrapper<EnvDeviceEquipment> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("device_code", one.getId()).orderByDesc("equipment_port");
            List<EnvDeviceEquipment> list = envDeviceEquipmentService.list(queryWrapper);
            String deviceStatus = "";
            if (list.size() != 0) {
                for (EnvDeviceEquipment envDeviceEquipment1 : list) {
                    deviceStatus = deviceStatus + envDeviceEquipment1.getUploadStart();
                }
                String s = deviceStatus;
                int i = Integer.parseInt(s, 2);
                jsonObject.put("dev_ctrl", i);
                log.info("设备配置完成开始发送：" + i);
                PublishMQTT publishMQTT = new PublishMQTT();
                publishMQTT.start("tcp://115.238.57.190:1883", "GUOWEIpublish", "admin", "admin", "huimv_down_" + chip, jsonObject.toJSONString());
            }
            log.info("---设备控制发送完成---");
        } else if ("dev_ctrl_suc".equals(messages)) {
            QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("device_code", chip);
            IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
            EnvDevice one = envDeviceService.getOne(queryWrapper);

            IEnvDeviceEquipmentService envDeviceEquipmentService = SpringUtil.getBean(IEnvDeviceEquipmentService.class);
            QueryWrapper<EnvDeviceEquipment> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_code", one.getId()).orderByDesc("equipment_port");
            List<EnvDeviceEquipment> list = envDeviceEquipmentService.list(queryWrapper1);
            if (list.size() != 0) {
                for (EnvDeviceEquipment equipment : list) {
                    equipment.setUpdateTime(new Date());
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    System.out.println("----这个是应答时间：" + sdf.format(new Date()));
                    envDeviceEquipmentService.updateById(equipment);
                    System.out.println("-----收到控制响应后修改标识成功！！");
                }
            }
            System.out.println("收到控制响应");
        } else {
            JSONObject jsonObject = JSON.parseObject(messages);
            System.out.println("这里是1111：" + jsonObject);
            if (ObjectUtil.isNotEmpty(jsonObject.get("alarm_event"))) {
                log.info("----报警信息开始保存---");
                JSONObject jsonObject1 = (JSONObject) jsonObject.get("alarm_event");
                try {
                    IEnvWarningInfoService envWarningInfoService = SpringUtil.getBean(IEnvWarningInfoService.class);
                    QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("device_code", chip);
                    IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
                    EnvDevice one = envDeviceService.getOne(queryWrapper);

                    EnvWarningInfo warningInfo = new EnvWarningInfo();
                    warningInfo.setWarningType(4);
                    warningInfo.setBuildLocation(one.getBuildLocation());
                    warningInfo.setUnitId(one.getUnitId());
                    warningInfo.setDate(new Date());
                    warningInfo.setFarmId(one.getFarmId());
                    Integer type = (Integer) jsonObject1.get("sta");
                    if (type == 1) {
                        warningInfo.setWarningContent(one.getBuildLocation() + "的设备断电报警！");
                    } else {
                        warningInfo.setWarningContent(one.getBuildLocation() + "的设备恢复供电！");
                    }
                    envWarningInfoService.save(warningInfo);
                } catch (Exception e) {
                    System.out.println("报警信息保存异常！！！");
                }

                log.info("----报警信息保存成功！----");
            } else if (ObjectUtil.isNotEmpty(jsonObject.get("sensor"))) {
                log.info("开始处理传感器数据！");
                IEnvDeviceService deviceService = SpringUtil.getBean(IEnvDeviceService.class);
                IEnvDeviceChipService chipService = SpringUtil.getBean(IEnvDeviceChipService.class);
                EnvDevice device = deviceService.getOne(new QueryWrapper<EnvDevice>().eq("device_type", 1).eq("device_code", chip));
                BigDecimal big = BigDecimal.valueOf(10);
                Map map = (Map) jsonObject.get("sensor");

                Map temp = (Map) map.get("temp");
                JSONArray jsonArray = (JSONArray) temp.get("data");
                BigDecimal data = new BigDecimal(Integer.parseInt(jsonArray.get(0).toString()));
                BigDecimal temp2 = data.divide(big);
                Map humi = (Map) map.get("humi");
                JSONArray jsonArray1 = (JSONArray) humi.get("data");
                BigDecimal data1 = new BigDecimal(Integer.parseInt(jsonArray1.get(0).toString()));
                BigDecimal humi2 = data1.divide(big);

                EnvData envData = new EnvData();
                if ("867699060005759".equals(chip) || "867699060004794".equals(chip)) {
                } else {
                    Map ph = (Map) map.get("ph");
                    JSONArray jsonArrayPH = (JSONArray) ph.get("data");
                    BigDecimal dataPH = new BigDecimal(Integer.parseInt(jsonArrayPH.get(0).toString()));
                    BigDecimal ph2 = dataPH.divide(big);
                    envData.setOther1(ph2.toString());
                }

                envData.setCreateTime(new Date());
                envData.setEnvTemp(temp2.toString());
                envData.setEnvHum(humi2.toString());
                //ph 值
                //除了9号舍有真实数据外 其他舍生成7.8-8


                envData.setDeviceId(device.getDeviceCode());
                envData.setUnitId(device.getUnitId());
                envData.setFarmId(device.getFarmId());
                IEnvDataService dataService = SpringUtil.getBean(IEnvDataService.class);
                if ("0".equals(humi2.toString()) || "0".equals(temp2.toString())){
                    return;
                }
                dataService.save(envData);
                log.info("温湿度处理完成！");


                //水表的值
                if ("867699060005759".equals(chip) || "867699060004794".equals(chip)) {
                    Map waterMeter = (Map) map.get("special_water_meter");
                    JSONArray jsonArray2 = (JSONArray) waterMeter.get("data");
                    JSONArray sta = (JSONArray) waterMeter.get("sta");
                    IEnergyWaterService waterService = SpringUtil.getBean(IEnergyWaterService.class);
                    QueryWrapper<EnvDeviceChip> queryWrapper = new QueryWrapper<>();
                    EnvDevice device1 = deviceService.getOne(new QueryWrapper<EnvDevice>().eq("device_type", 1).eq("device_code", chip));
                    queryWrapper.eq("device_code", device1.getDeviceCode()).eq("chip_type", 3);
                    List<EnvDeviceChip> chips = chipService.list(queryWrapper);
//                IEnvDeviceChipService chipService = SpringUtil.getBean(IEnvDeviceChipService.class);
                    for (int i = 0; i < jsonArray2.size(); i++) {
                        BigDecimal big1 = BigDecimal.valueOf(1000);
                        QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", device1.getFarmId()).orderByDesc("id").eq("chip_code", 1006)
                                .last(" limit 1");
                        if ("867699060005759".equals(chip)) {
                            queryWrapper1.eq("unit_id", 9942);
                        } else {
                            queryWrapper1.eq("unit_id", 9943);
                        }
                        EnergyWater water1 = waterService.getOne(queryWrapper1);
                        EnergyWater water = new EnergyWater();
                        BigDecimal data2 = new BigDecimal(Integer.parseInt(jsonArray2.get(i).toString()));
                        BigDecimal waterMeter2 = data2.divide(big1);

                        if (sta.get(0).equals("4")||waterMeter2.equals(BigDecimal.ZERO)) {
                            water.setWaterValue("0");
                            if (ObjectUtil.isNotEmpty(water1)) {
                                water.setOriginalValue(water1.getOriginalValue());
                            } else {
                                water.setOriginalValue("0");
                            }
                        }else {
                            if (ObjectUtil.isNotEmpty(water1)) {
                                BigDecimal bd = new BigDecimal(water1.getOriginalValue());
                                    if (waterMeter2.compareTo(bd) <= 0) {
                                        water.setOriginalValue(water1.getOriginalValue());
                                        water.setWaterValue("0");
                                    } else {
                                        water.setOriginalValue(waterMeter2.toString());
                                        water.setWaterValue(waterMeter2.subtract(bd).toString());
                                    }

                            } else {
                                water.setWaterValue(waterMeter2.toString());
                                water.setOriginalValue(waterMeter2.toString());
                            }
                        }

                        water.setCreateDate(new Date());
                        water.setFarmId(device1.getFarmId());
                        water.setUnitId(device1.getUnitId());
                        water.setChipCode(chips.get(i).getChipCode());
                        waterService.save(water);
                    }
                    log.info("水表处理完成！");
                } else {
                    Map waterMeter = (Map) map.get("water_meter");
                    JSONArray jsonArray2 = (JSONArray) waterMeter.get("data");
                    IEnergyWaterService waterService = SpringUtil.getBean(IEnergyWaterService.class);
                    QueryWrapper<EnvDeviceChip> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("device_code", device.getDeviceCode()).eq("chip_type", 3);
                    List<EnvDeviceChip> chips = chipService.list(queryWrapper);
//                IEnvDeviceChipService chipService = SpringUtil.getBean(IEnvDeviceChipService.class);
                    for (int i = 0; i < jsonArray2.size(); i++) {
                        QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("farm_id", device.getFarmId()).orderByDesc("id").eq("chip_code", chips.get(i).getChipCode())
                                .last(" limit 1");
                        EnergyWater water1 = waterService.getOne(queryWrapper1);
                        EnergyWater water = new EnergyWater();
                        BigDecimal data2 = new BigDecimal(Integer.parseInt(jsonArray2.get(i).toString()));
                        BigDecimal waterMeter2 = data2.divide(big);
                        if (ObjectUtil.isNotEmpty(water1)) {
                            if ("".equals(water1.getOriginalValue()) || water1.getOriginalValue() == null) {
                                water.setOriginalValue(waterMeter2.toString());
                                water.setWaterValue(waterMeter2.toString());
                            } else {
                                water.setOriginalValue(waterMeter2.toString());
                                BigDecimal bd = new BigDecimal(water1.getOriginalValue());
                                if (bd.compareTo(BigDecimal.ZERO) == -1) {
                                    water.setWaterValue(BigDecimal.ZERO.toString());
                                } else {
                                    water.setWaterValue(waterMeter2.subtract(bd).toString());
                                }
                            }
                        } else {
                            water.setWaterValue(waterMeter2.toString());
                            water.setOriginalValue(waterMeter2.toString());
                        }
                        water.setCreateDate(new Date());
                        water.setFarmId(device.getFarmId());
                        water.setUnitId(device.getUnitId());
                        water.setChipCode(chips.get(i).getChipCode());
                        waterService.save(water);
                    }
                    log.info("水表处理完成！");
                }


                //电表的值
                if ("867699060005759".equals(chip) || "867699060004794".equals(chip)) {

                } else {
                    Map elect = (Map) map.get("electr_meter");
                    JSONArray jsonArray3 = (JSONArray) elect.get("data");
                    IEnergyElectricityService electricityService = SpringUtil.getBean(IEnergyElectricityService.class);

                    QueryWrapper<EnvDeviceChip> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("device_code", device.getDeviceCode()).eq("chip_type", 4);
                    List<EnvDeviceChip> chips2 = chipService.list(queryWrapper1);

                    for (EnvDeviceChip deviceChip : chips2) {
                        QueryWrapper<EnergyElectricity> queryWrapper2 = new QueryWrapper<>();
                        queryWrapper2.eq("farm_id", device.getFarmId()).orderByDesc("id").eq("chip_code", deviceChip.getChipCode())
                                .last(" limit 1");
                        EnergyElectricity water1 = electricityService.getOne(queryWrapper2);
                        EnergyElectricity electricity = new EnergyElectricity();
                        BigDecimal data2 = new BigDecimal(Integer.parseInt(jsonArray3.get(0).toString()));
                        BigDecimal elect2 = data2.divide(big);
                        if (ObjectUtil.isNotEmpty(water1)) {
                            if ("".equals(water1.getOriginalValue()) || water1.getOriginalValue() == null) {
                                electricity.setOriginalValue(elect2.toString());
                                electricity.setElectricityValue(elect2.toString());
                            } else {
                                electricity.setOriginalValue(elect2.toString());
                                BigDecimal bd = new BigDecimal(water1.getOriginalValue());
                                electricity.setElectricityValue(elect2.subtract(bd).toString());
                            }
                        } else {
                            electricity.setElectricityValue(elect2.toString());
                            electricity.setOriginalValue(elect2.toString());
                        }
                        electricity.setCreateDate(new Date());
                        electricity.setUnitId(device.getUnitId());
                        electricity.setFarmId(device.getFarmId());
                        electricity.setChipCode(deviceChip.getChipCode());
                        electricityService.save(electricity);
                    }

                    log.info("电表处理完成！");
                }




                //水压的值
                BigDecimal big2 = BigDecimal.valueOf(100);
                Map waterPress = (Map) map.get("water_press");
                JSONArray jsonArray4 = (JSONArray) waterPress.get("data");
                QueryWrapper<EnvDeviceChip> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("device_code", device.getDeviceCode()).eq("chip_type", 5);
                List<EnvDeviceChip> chips3 = chipService.list(queryWrapper2);

                IEnergyWaterPressureService pressureService = SpringUtil.getBean(IEnergyWaterPressureService.class);
                for (int i = 0; i < jsonArray4.size(); i++) {
                    EnergyWaterPressure waterPressure = new EnergyWaterPressure();
                    BigDecimal data3 = new BigDecimal(Integer.parseInt(jsonArray4.get(i).toString()));
                    BigDecimal waterMeter2 = data3.divide(big2);
                    waterPressure.setWaterValue(waterMeter2.toString());
                    waterPressure.setFarmId(device.getFarmId());
                    waterPressure.setUnitId(device.getUnitId());
                    waterPressure.setChipCode(chips3.get(i).getChipCode());
                    waterPressure.setCreateDate(LocalDateTime.now());
                    pressureService.save(waterPressure);
                    log.info("水压处理完成！");
                }
            }

            System.out.println("数据处理完成！");
        }

    }
}
