package com.huimv.guowei.admin.mqtt.moveScale;

import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

@Configuration
@Component
public class SendData  {

    public static final String HOST = "tcp://115.238.57.190:1883";
    private static final String clientid = "GuoWeiSend";
    private MqttClient client;
    private MqttConnectOptions options;

    private String userName = "admin";    //非必须
    private String passWord = "admin";  //非必须

    public void start() {
        try {
            // host为主机名，clientid即连接MQTT的客户端ID，一般以唯一标识符表示，MemoryPersistence设置clientid的保存形式，默认为以内存保存
            client = new MqttClient(HOST, clientid, new MemoryPersistence());
            // MQTT的连接设置
            options = new MqttConnectOptions();
            // 设置是否清空session,这里如果设置为false表示服务器会保留客户端的连接记录，设置为true表示每次连接到服务器都以新的身份连接
            options.setCleanSession(true);
            // 设置连接的用户名
            options.setUserName(userName);
            // 设置连接的密码
            options.setPassword(passWord.toCharArray());
            // 设置超时时间 单位为秒
            options.setConnectionTimeout(10);
            // 设置会话心跳时间 单位为秒 服务器会每隔1.5*20秒的时间向客户端发送个消息判断客户端是否在线，但这个方法并没有重连的机制
            options.setKeepAliveInterval(20);
            //设置断开后重新连接
            options.setAutomaticReconnect(true);
            // 设置回调

            String s = "02 52 03";
            byte[] bytes = hexStr2Bytes(s);
            client.connect(options);
            pubMessage(bytes, "huimv_GW_up");
            System.out.println("发送完成！");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 消息发送
     *
     * @param message
     * @param topic
     */
    public void pubMessage(byte[] message, String topic) {
        MqttMessage mess = new MqttMessage();
        mess.setQos(0);
        mess.setRetained(true);
        mess.setPayload(message);
        System.out.println(mess);
        try {
            client.publish(topic, mess);
            client.disconnect();
        } catch (Exception e) {
            //LOGGER.error(e.getLocalizedMessage());
        }
    }

    /**
     * @param src
     * @return byte[]
     * @Description: 字符串转byte数组
     * @Title: hexStr2Bytes
     * @author qin
     * @date 2021年11月30日下午5:31:30
     */
    public static byte[] hexStr2Bytes(String src) {
        src = src.replaceAll(" ", "");
        System.out.println(src);
        int m = 0, n = 0;
        int l = src.length() / 2;
        System.out.println(l);
        byte[] ret = new byte[l];
        for (int i = 0; i < l; i++) {
            m = i * 2 + 1;
            n = m + 1;
            String sss = "0x" + src.substring(i * 2, m) + src.substring(m, n);
            System.out.println("sss[" + i + "]:" + sss);
            try {
                ret[i] = Byte.decode(sss);
            } catch (Exception e) {
                // TODO: handle exception
                int s = Integer.decode(sss);
                ret[i] = (byte) s;
            }
        }
        return ret;
    }
}

