package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.common.utils.UploadImage;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.entity.vo.*;
import com.huimv.guowei.admin.mapper.*;
import com.huimv.guowei.admin.service.IBaseDuckInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import com.huimv.guowei.admin.utils.Print;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class BaseDuckInfoServiceImpl extends ServiceImpl<BaseDuckInfoMapper, BaseDuckInfo> implements IBaseDuckInfoService {

    @Autowired
    private BaseDuckInfoMapper duckInfoMapper;
    @Autowired
    private EnvMoveCallMapper moveCallMapper;
    @Autowired
    private EnvRegularCallFeedingMapper feedingMapper;
    @Autowired
    private EnvRegularCallEggMapper eggMapper;

    @Autowired
    private UploadImage uploadImage;
    @Autowired
    private BaseDuckBreedImgMapper imgMapper;
    @Autowired
    private DepartureMapper departureMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;
    @Autowired
    private BatchMapper batchMapper;


    @Override
    public Result listDuck(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String chiNum = paramsMap.get("chiNum");//前端传来的是编号
        String location = paramsMap.get("location");
        String batchNum = paramsMap.get("batchNum");
        String farmId = paramsMap.get("farmId");//场的编码
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);

        if (ObjectUtil.isNotEmpty(chiNum) && ObjectUtil.isNotEmpty(batchNum)) {
            queryWrapper.like(StringUtils.isNotBlank(batchNum), "batch_num", batchNum)
                    .like(StringUtils.isNotBlank(location), "unit_name", location)
                    .and(wrapper -> wrapper.like(StringUtils.isNotBlank(chiNum), "chi_num", chiNum)
                            .or().like(StringUtils.isNotBlank(chiNum), "jiao_num", chiNum))
                    .orderByDesc("id");
        } else {
            queryWrapper.like(StringUtils.isNotBlank(batchNum), "batch_num", batchNum)
                    .like(StringUtils.isNotBlank(chiNum), "chi_num", chiNum)
                    .or().like(StringUtils.isNotBlank(chiNum), "jiao_num", chiNum)
                    .like(StringUtils.isNotBlank(location), "unit_name", location)
                    .orderByDesc("id");
        }
        Page<BaseDuckInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, duckInfoMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result addDuck(HttpServletRequest httpServletRequest, Map<String, String> paramsMap, MultipartFile imgUrl) throws ParseException, IOException {
        BaseDuckInfo baseDuckInfo = new BaseDuckInfo();
        QueryWrapper<BaseDuckInfo> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("farm_id", paramsMap.get("farmId")).orderByDesc("id").last(" limit 1");
        BaseDuckInfo info = duckInfoMapper.selectOne(queryWrapper3);
        if (ObjectUtil.isNotEmpty(info)) {
            baseDuckInfo.setDuckNum(String.valueOf(Integer.parseInt(info.getDuckNum()) + 1));
        } else {
            baseDuckInfo.setDuckNum("20231124");
        }
        baseDuckInfo.setBatchNum(paramsMap.get("batchNum"));
        baseDuckInfo.setChiNum(paramsMap.get("chiNum"));
        if (StringUtils.isNotBlank(paramsMap.get("jiaoNum"))) {
            baseDuckInfo.setJiaoNum(paramsMap.get("jiaoNum"));
        }
        if (StringUtils.isNotBlank(paramsMap.get("unitList"))) {
            baseDuckInfo.setUnitList(paramsMap.get("unitList"));
        }
        if (StringUtils.isNotBlank(paramsMap.get("unitName"))) {
            baseDuckInfo.setUnitName(paramsMap.get("unitName"));
        }
        if (StringUtils.isNotBlank(paramsMap.get("duckSex"))) {
            baseDuckInfo.setDuckSex(Integer.parseInt(paramsMap.get("duckSex")));
        }


        baseDuckInfo.setFarmId(Integer.parseInt(paramsMap.get("farmId")));
        baseDuckInfo.setDuckBreed(Integer.parseInt(paramsMap.get("duckBreed")));
        baseDuckInfo.setDuckBirthplace(paramsMap.get("duckBirthPlace"));
        String date = paramsMap.get("duckBirthDay");
        date = date + " 00:00:00";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd ");
        baseDuckInfo.setDuckBirthday(sdf.parse(date));

        Date date1 = sdf.parse(date);
        Date date2 = new Date();
        // 计算两个日期之间的天数
        int between = daysBetween(date1, date2);
        baseDuckInfo.setDayAge(between);
        baseDuckInfo.setGenotype(paramsMap.get("genotype"));
        BigDecimal big = new BigDecimal(paramsMap.get("nowWeight"));
        baseDuckInfo.setNowWeight(big);
        if (StringUtils.isNotBlank(paramsMap.get("unitId"))) {
            baseDuckInfo.setUnitId(Integer.parseInt(paramsMap.get("unitId")));
        }
        if (StringUtils.isNotBlank(paramsMap.get("breedEnv"))) {
            baseDuckInfo.setBreedingEnv(paramsMap.get("breedEnv"));
        }


        String fatherNum = paramsMap.get("fatherNum");
        String motherNum = paramsMap.get("motherNum");
        String duckCode = paramsMap.get("duckCode");
        baseDuckInfo.setFatherChiNum(paramsMap.get("fatherChiNum"));
        baseDuckInfo.setFatherJiaoNum(paramsMap.get("fatherJiaoNum"));
        baseDuckInfo.setMotherChiNum(paramsMap.get("motherChiNum"));
        baseDuckInfo.setMotherJiaoNum(paramsMap.get("motherJiaoNum"));
        if (StringUtils.isNotBlank(fatherNum) && StringUtils.isNotBlank(motherNum)) {
            if (fatherNum.equals(motherNum) ||
                    motherNum.equals(duckCode) ||
                    fatherNum.equals(duckCode)) {
                return new Result(10001, "鸭只编号、父亲编号、母亲编号三者的编号不能相同！", false);
            }
            QueryWrapper<BaseDuckInfo> duckInfoQueryWrapper = new QueryWrapper<>();
            duckInfoQueryWrapper.eq("farm_id", paramsMap.get("farmId")).eq("duck_num", fatherNum).eq("is_cage", 0);
            BaseDuckInfo father = duckInfoMapper.selectOne(duckInfoQueryWrapper);//父亲编号

            if (ObjectUtil.isEmpty(father)) {
                return new Result(10001, "父亲编号不存在，请重新输入！", false);
            } else {
                if (father.getDuckSex() == 1) {
                    return new Result(10001, "该编号为母亲编号，请重新填写！", false);
                }
                if (father.getDayAge() < 100) {
                    return new Result(10001, "父亲（或母亲）鸭只日龄小于100天", false);
                } else {
                    QueryWrapper<BaseDuckInfo> duckInfoQueryWrapper1 = new QueryWrapper<>();
                    duckInfoQueryWrapper1.eq("farm_id", paramsMap.get("farmId")).eq("duck_num", motherNum).eq("is_cage", 0);
                    BaseDuckInfo mother = duckInfoMapper.selectOne(duckInfoQueryWrapper1);//母亲编号
                    if (ObjectUtil.isEmpty(mother)) {
                        return new Result(10001, "母亲编号不存在，请重新输入！", false);
                    } else {
                        if (mother.getDuckSex() == 0) {
                            return new Result(10001, "该编号为父亲编号，请重新填写！", false);
                        }

                        if (mother.getDayAge() < 100) {
                            return new Result(10001, "父亲（或母亲）鸭只日龄小于100天！", false);
                        } else {
                            if (StringUtils.isNotBlank(paramsMap.get("fatherNum"))) {
                                baseDuckInfo.setFatherNum(paramsMap.get("fatherNum"));
                                baseDuckInfo.setMotherNum(paramsMap.get("motherNum"));
                                baseDuckInfo.setFatherGenotype(father.getGenotype());
                                baseDuckInfo.setMotherGenotype(mother.getGenotype());
                            }
                        }
                    }
                }
            }
        }

        if (StringUtils.isBlank(fatherNum) && StringUtils.isNotBlank(motherNum)) {
            return new Result(10001, "母亲编号已填写，请输入父亲编号！", false);
        }
        if (StringUtils.isBlank(motherNum) && StringUtils.isNotBlank(fatherNum)) {
            return new Result(10001, "父亲编号已填写，请输入母亲编号！", false);
        }
        if (StringUtils.isNotBlank(paramsMap.get("specialNeeds"))) {
            baseDuckInfo.setSpecialNeeds(paramsMap.get("specialNeeds"));
        }
        baseDuckInfo.setIsCage(0);
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("chi_num", paramsMap.get("chiNum")).eq("farm_id", paramsMap.get("farmId"));
        Integer integer = duckInfoMapper.selectCount(queryWrapper);
        if (StringUtils.isNotBlank(paramsMap.get("jiaoNum"))) {
            QueryWrapper<BaseDuckInfo> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("jiao_num", paramsMap.get("jiaoNum")).eq("farm_id", paramsMap.get("farmId"));
            Integer integer1 = duckInfoMapper.selectCount(queryWrapper2);
            if (integer1 != 0) {
                return new Result(10001, "鸭只脚号已存在", false);
            }
        }
        if (integer == 0) {
            if (StringUtils.isNotBlank(paramsMap.get("unitId"))) {
                QueryWrapper<BaseDuckInfo> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("unit_id", paramsMap.get("unitId"));
                List<BaseDuckInfo> baseDuckInfos = duckInfoMapper.selectList(queryWrapper1);
                for (BaseDuckInfo duckInfo : baseDuckInfos) {
                    if (duckInfo.getIsCage() == 0) {
                        return new Result(10001, "鸭笼已被使用！", false);
                    }
                }
            }

            if (ObjectUtil.isEmpty(imgUrl)) {
                String duckBreed = paramsMap.get("duckBreed");
                if (StringUtils.isNotBlank(paramsMap.get("duckSex"))) {
                    if (Integer.parseInt(duckBreed) == 3 && Integer.parseInt(paramsMap.get("duckSex")) == 1) {
                        duckBreed = "10";
                    }
                }
                QueryWrapper<BaseDuckBreedImg> imgQueryWrapper = new QueryWrapper<>();
                imgQueryWrapper.eq("duck_breed", duckBreed);
                BaseDuckBreedImg breedImg = imgMapper.selectOne(imgQueryWrapper);
                baseDuckInfo.setDuckImgUrl(breedImg.getImgUrl());
            } else {
                String content = uploadImage.getImageCom(imgUrl);
                baseDuckInfo.setDuckImgUrl(content);
            }
            duckInfoMapper.insert(baseDuckInfo);
            return new Result(10000, "添加成功！", true);
        } else {
            return new Result(10001, "鸭只翅号已存在", false);
        }

    }


    /**
     * 获取两个日期间的天数，精确到年月日
     *
     * @param start
     * @param end
     * @return
     */
    public static int daysBetween(Date start, Date end) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            start = sdf.parse(sdf.format(start));
            end = sdf.parse(sdf.format(end));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        long startTime = start.getTime();
        long endTime = end.getTime();
        long betweenDays = (endTime - startTime) / (1000 * 3600 * 24);
        return Integer.parseInt(String.valueOf(betweenDays));
    }

    @Override
    public Result editDuck(HttpServletRequest httpServletRequest, BaseDuckInfo baseDuckInfo) {

        Date date = baseDuckInfo.getDuckBirthday();
        Date date2 = new Date();
        // 计算两个日期之间的天数
        int between = daysBetween(date, date2);
        baseDuckInfo.setDayAge(between);

        String fatherNum = baseDuckInfo.getFatherNum();
        String motherNum = baseDuckInfo.getMotherNum();
        String duckCode = baseDuckInfo.getDuckNum();
        if (StringUtils.isNotBlank(fatherNum) && StringUtils.isNotBlank(motherNum)) {
            if (fatherNum.equals(motherNum) ||
                    motherNum.equals(duckCode) ||
                    fatherNum.equals(duckCode)) {
                return new Result(10001, "鸭只编号、父亲编号、母亲编号三者的编号不能相同！", false);
            }
            QueryWrapper<BaseDuckInfo> duckInfoQueryWrapper = new QueryWrapper<>();
            duckInfoQueryWrapper.eq("farm_id", baseDuckInfo.getFarmId()).eq("duck_num", fatherNum).eq("is_cage", 0);
            BaseDuckInfo father = duckInfoMapper.selectOne(duckInfoQueryWrapper);//父亲编号

            if (ObjectUtil.isEmpty(father)) {
                return new Result(10001, "父亲编号不存在，请重新输入！", false);
            } else {
                if (father.getDuckSex() == 1) {
                    return new Result(10001, "该编号为母亲编号，请重新填写！", false);
                }

                if (father.getDayAge() < 100) {
                    return new Result(10001, "父亲（或母亲）鸭只日龄小于100天", false);
                } else {
                    QueryWrapper<BaseDuckInfo> duckInfoQueryWrapper1 = new QueryWrapper<>();
                    duckInfoQueryWrapper1.eq("farm_id", baseDuckInfo.getFarmId()).eq("duck_num", motherNum).eq("is_cage", 0);
                    BaseDuckInfo mother = duckInfoMapper.selectOne(duckInfoQueryWrapper1);//母亲编号
                    if (ObjectUtil.isEmpty(mother)) {
                        return new Result(10001, "母亲编号不存在，请重新输入！", false);
                    } else {
                        if (mother.getDuckSex() == 0) {
                            return new Result(10001, "该编号为父亲编号，请重新填写！", false);
                        }

                        if (mother.getDayAge() < 100) {
                            return new Result(10001, "父亲（或母亲）鸭只日龄小于100天！", false);
                        }
                        baseDuckInfo.setFatherGenotype(father.getGenotype());
                        baseDuckInfo.setMotherGenotype(mother.getGenotype());
                    }
                }
            }
        }


        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.ne("id", baseDuckInfo.getId()).
                and(i -> i.eq("duck_num", baseDuckInfo.getDuckNum()).or().eq("unit_id", baseDuckInfo.getUnitId()));
        List<BaseDuckInfo> baseDuckInfos = duckInfoMapper.selectList(queryWrapper);
        if (baseDuckInfos.size() == 0) {
            duckInfoMapper.updateById(baseDuckInfo);
            return new Result(10000, "修改成功！", true);
        } else {
            for (BaseDuckInfo duckInfo : baseDuckInfos) {
                if (duckInfo.getIsCage() == 0) {
                    return new Result(10001, "鸭只编号已存在或者鸭笼已被占用", false);
                }
                if (duckInfo.getDuckNum() == baseDuckInfo.getDuckNum()) {
                    return new Result(10001, "鸭只编号已存在或者鸭笼已被占用", false);
                }
            }
            duckInfoMapper.updateById(baseDuckInfo);
            return new Result(10000, "修改成功！", true);
        }

    }

    @Override
    public Result listDuckById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        BaseDuckInfo baseDuckInfo = duckInfoMapper.selectOne(queryWrapper);
        return new Result(ResultCode.SUCCESS, baseDuckInfo);
    }

    @Override
    public void printDuck(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String duckCodes = paramsMap.get("duckCode");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");
        String[] split = duckCodes.split(",");

        List<PrintEntrty> list = new ArrayList<>();
        DecimalFormat def = new DecimalFormat("0.00");

        for (String duckCode : split) {
            PrintEntrty printEntrty = new PrintEntrty();
            QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_num", duckCode);
            BaseDuckInfo baseDuckInfo = duckInfoMapper.selectOne(queryWrapper);//鸭只档案

//            QueryWrapper<EnvMoveCall> queryWrapper1 = new QueryWrapper<>();
//            queryWrapper1.eq("chi_num", baseDuckInfo.getChiNum()).
//                    between(StringUtils.isNotBlank(startTime), "call_date", startTime, endTime);
            List<EnvMoveCall> envMoveCalls = moveCallMapper.listPrint(baseDuckInfo.getChiNum(), startTime, endTime);//日龄体重

            QueryWrapper<EnvRegularCallFeeding> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("chi_num", baseDuckInfo.getChiNum()).
                    between(StringUtils.isNotBlank(startTime), "call_date", startTime + " 00:00:00", endTime + " 23:59:59");
            List<EnvRegularCallFeeding> feedings = feedingMapper.selectList(queryWrapper2);//采食
            double count = 0;
            for (EnvRegularCallFeeding feeding : feedings) {
                count = count + feeding.getDuckWeight();
            }

            Date resultDate = null;
            int totalEgg = 0;
            Date duckBirthday = baseDuckInfo.getDuckBirthday();
            DateTime date = DateUtil.offsetDay(duckBirthday, 298);
            Date dateTime = new Date(date.getTime());
            List<EnvRegularCallEgg> envRegularCallEggs = eggMapper.selectList(new QueryWrapper<EnvRegularCallEgg>().lambda()
                    .eq(EnvRegularCallEgg::getDuckNum, baseDuckInfo.getDuckNum())
                    .ge(EnvRegularCallEgg::getCallDate, dateTime));
            if (ObjectUtil.isNotEmpty(envRegularCallEggs)) {
                for (EnvRegularCallEgg envRegularCallEgg : envRegularCallEggs) {
                    totalEgg += envRegularCallEgg.getEggNum();
                    if (totalEgg > 3) {
                        resultDate = envRegularCallEgg.getCallDate();
                        System.out.println("满足日龄的日期为:" + resultDate);
                        break;
                    }
                }
            }
            printEntrty.setBaseDuckInfo(baseDuckInfo);
            printEntrty.setFeeding(def.format(count));
            Date endDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(endTime + " 23:59:59");
            Date startDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(startTime + " 00:00:00");
            System.out.println("开始时间:" + startDate);
            System.out.println("结束时间:" + endDate);
            System.out.println("满足日龄日期:" + resultDate);
            if (ObjectUtil.isNotEmpty(resultDate)) {
                if (resultDate.after(endDate)) {
                    System.out.println("满足日龄在结束日期之后");
                    printEntrty.setEggNum("0");
                    printEntrty.setEgging("0");
                    printEntrty.setAvgWeiht("0");
                } else {
                    QueryWrapper<EnvRegularCallEgg> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper.eq("duck_num", baseDuckInfo.getDuckNum());
                    EggVo eggVo = new EggVo();
                    if (resultDate.before(startDate)) {
                        queryWrapper1.between("call_date", startDate, endDate);
                        System.out.println("满足日龄在开始日期之前");
                        eggVo = eggMapper.print(queryWrapper1);
                    } else {
                        queryWrapper1.between("call_date", resultDate, endDate);
                        System.out.println("满足日龄在开始日期之后");
                        eggVo = eggMapper.print(queryWrapper1);
                    }
                    printEntrty.setEggNum(eggVo.getTotalEgg());
                    printEntrty.setEgging(eggVo.getTotalWeight());
                    printEntrty.setAvgWeiht(eggVo.getAvgWeight());
                }
            } else {
                System.out.println("日龄不满足");
                printEntrty.setEggNum("0");
                printEntrty.setEgging("0");
                printEntrty.setAvgWeiht("0");
            }

            printEntrty.setEnvMoveCalls(envMoveCalls);
            list.add(printEntrty);
        }

        Print.printDuck(list);
//        Print.printDuck(baseDuckInfo, duckHealthInfos, duckImmunityInfos, envMoveCalls);
    }


    @Override
    public void printChi(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("is_cage", 0);
        List<BaseDuckInfo> baseDuckInfos = duckInfoMapper.selectList(queryWrapper);
        Print.printChi(baseDuckInfos);
    }

    @Override
    public void updateDayAge() {
        duckInfoMapper.updateDayAge();
    }

    @Transactional
    @Override
    public Result importData(HttpServletRequest httpServletRequest, Map<String, String> paramsMap, MultipartFile files) throws Exception {
        String farmId = paramsMap.get("farmId");

        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("翅号", "chiNum");
        map.put("脚号", "jiaoNum");
        map.put("代次号", "batchNum");
        map.put("性别", "duckSex");
        map.put("品种", "duckBreed");
        map.put("来源", "duckBirthplace");
        map.put("基因型", "genotype");
        map.put("出生日期", "duckBirthday");
        map.put("位置", "unitName");
        map.put("出生体重", "nowWeight");
        map.put("饲养环境", "breedingEnv");
        map.put("特殊需求", "specialNeeds");
        map.put("父亲翅号", "fatherChiNum");
        map.put("父亲脚号", "fatherJiaoNum");
        map.put("母亲翅号", "motherChiNum");
        map.put("母亲脚号", "motherJiaoNum");
        try (
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<BaseDuckInfoImportData> list = ExcelImportSheet.getListFromExcel(sheet, BaseDuckInfoImportData.class, map);

            for (BaseDuckInfoImportData breed : list) {
                //底层数据库操作 insert什么的
                if (StringUtils.isNotBlank(breed.getUnitName())) {
                    QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("farm_id", farmId).eq("build_name", breed.getUnitName());
                    BaseBuilding baseBuilding = buildingMapper.selectOne(queryWrapper);
                    Integer integer = baseMapper.selectCount(new QueryWrapper<BaseDuckInfo>().lambda()
                            .eq(BaseDuckInfo::getUnitId, baseBuilding.getId()).eq(BaseDuckInfo::getFarmId, farmId)
                            .eq(BaseDuckInfo::getIsCage, 0).ne(BaseDuckInfo::getChiNum, breed.getChiNum()));
                    if (integer > 0) {
                        return new Result(10001, breed.getUnitName() + "已存在鸭只!", false);
                    }
                    breed.setUnitId(baseBuilding.getId());
                    breed.setUnitList(baseBuilding.getParentId() + "," + baseBuilding.getId());
                }

                if (StringUtils.isNotBlank(breed.getBatchNum())) {
                    Batch batch = batchMapper.selectOne(new QueryWrapper<Batch>().lambda().eq(Batch::getBatchNum, breed.getBatchNum()).eq(Batch::getFarmId, farmId).eq(Batch::getIsEnable, 0));
                    if (ObjectUtil.isEmpty(batch)) {
                        return new Result(10001, "代次号" + breed.getBatchNum() + "不存在或未启用!", false);
                    }
                }

                breed.setFarmId(Integer.parseInt(farmId));
                if (StringUtils.isNotBlank(breed.getDuckSex())) {
                    if ("雄性".equals(breed.getDuckSex())) {
                        breed.setDuckSex("0");
                    }
                    if ("雌性".equals(breed.getDuckSex())) {
                        breed.setDuckSex("1");
                    }
                }
                QueryWrapper<BaseDuckBreedImg> queryWrapper1 = new QueryWrapper<>();
                if ("北京鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 1);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("1");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("连城白鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 2);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("2");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("山麻鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 3);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("3");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("攸县麻鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 4);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("4");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("白改鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 5);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("5");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("缙云麻鸭".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 6);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("6");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("绍兴鸭(带圈白翼梢)".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 7);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("7");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("绍兴鸭(红毛绿翼梢)".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 8);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("8");
                    breed.setDuckImgUrl(img.getImgUrl());
                } else if ("绍兴鸭(白羽绍鸭)".equals(breed.getDuckBreed())) {
                    queryWrapper1.eq("duck_breed", 9);
                    BaseDuckBreedImg img = imgMapper.selectOne(queryWrapper1);
                    breed.setDuckBreed("9");
                    breed.setDuckImgUrl(img.getImgUrl());
                }
                QueryWrapper<BaseDuckInfo> queryWrapper3 = new QueryWrapper<>();
                queryWrapper3.eq("farm_id", farmId).orderByDesc("id").last(" limit 1");
                BaseDuckInfo info = duckInfoMapper.selectOne(queryWrapper3);
                if (ObjectUtil.isNotEmpty(info)) {
                    breed.setDuckNum(String.valueOf(Integer.parseInt(info.getDuckNum()) + 1));
                } else {
                    breed.setDuckNum("20231124");
                }


                BaseDuckInfo baseDuckInfo = new BaseDuckInfo();
                BeanUtil.copyProperties(breed, baseDuckInfo);

                Date birthday = breed.getDuckBirthday();
                Date date2 = new Date();
                int result = birthday.compareTo(date2);
                if (result > 0) {
                    return new Result(10001, "出生日期不能超过当前时间", false);
                }
                // 计算两个日期之间的天数
                int between = daysBetween(birthday, date2);
                baseDuckInfo.setDayAge(between);
                if (StringUtils.isNotBlank(breed.getDuckSex())) {
                    baseDuckInfo.setDuckSex(Integer.parseInt(breed.getDuckSex()));
                }
                if ("".equals(breed.getJiaoNum()) || null == breed.getJiaoNum()) {
                    baseDuckInfo.setJiaoNum(null);
                }
                baseDuckInfo.setDuckBreed(Integer.parseInt(breed.getDuckBreed()));
                baseDuckInfo.setIsCage(0);
                //翅号存在就更新
                BaseDuckInfo baseDuckInfo1 = duckInfoMapper.selectOne(new QueryWrapper<BaseDuckInfo>().lambda().eq(BaseDuckInfo::getChiNum, breed.getChiNum()).eq(BaseDuckInfo::getFarmId, farmId));
                if (ObjectUtil.isNotEmpty(baseDuckInfo1)) {
                    duckInfoMapper.update(baseDuckInfo, new UpdateWrapper<BaseDuckInfo>().lambda().eq(BaseDuckInfo::getChiNum, baseDuckInfo.getChiNum()).eq(BaseDuckInfo::getFarmId, farmId));
                } else {
                    duckInfoMapper.insert(baseDuckInfo);
                }
                System.out.println(breed.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        } finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }

    @Override
    public Result listDuckByScreen(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckNum = paramsMap.get("duckNum");//鸭只编号
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("duck_num", duckNum).eq("farm_id", farmId);
        BaseDuckInfo baseDuckInfo = duckInfoMapper.selectOne(queryWrapper);//鸭只档案
        BaseDuckInfoVo baseDuckInfoVo = new BaseDuckInfoVo();
        BeanUtil.copyProperties(baseDuckInfo, baseDuckInfoVo);

        QueryWrapper<Departure> departureQueryWrapper = new QueryWrapper<>();
        departureQueryWrapper.eq("duck_num", baseDuckInfo.getDuckNum());
        Departure departure = departureMapper.selectOne(departureQueryWrapper);//离场

        QueryWrapper<EnvRegularCallEgg> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).eq("duck_num", duckNum);
        queryWrapper2.select(" IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum");
        EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper2);//产蛋数量

        if (ObjectUtil.isEmpty(departure)) {
            Date birthday = baseDuckInfo.getDuckBirthday();
            DateTime day = DateUtil.offsetDay(birthday, 504);
            baseDuckInfoVo.setOutTime(day);
        } else {
            baseDuckInfoVo.setOutTime(departure.getDate());
        }

        if (ObjectUtil.isEmpty(egg)) {
            baseDuckInfoVo.setEggNum(0);
        } else {
            baseDuckInfoVo.setEggNum(egg.getEggNum());
        }

        /*Date timesmorning = DataUill.getTimesmorning();*/
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).ne("duck_weight", 0.0).eq("duck_num", duckNum)/*.ge("call_date",timesmorning)*/;
        List<EnvRegularCallFeeding> feedings = feedingMapper.listDayDesc(queryWrapper1);
        JSONArray jsonArray = new JSONArray();
        for (EnvRegularCallFeeding feeding : feedings) {
            String s = feeding.getChiNum() + "翅号鸭进食" + feeding.getDuckWeight() + "g,";
            Date callDate = feeding.getCallDate();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String s1 = sdf.format(callDate);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("time", s1);
            jsonObject.put("info", s);
            jsonArray.add(jsonObject);
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("duckInfo", baseDuckInfoVo);
        jsonObject.put("feedInfo", jsonArray);
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listDuckByScreen2(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String duckNum = paramsMap.get("duckNum");
        DateTime dateTime = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -7);
        JSONObject jsonObject = new JSONObject();

        List<EnvMoveCallVo> envMoveCallVoList = new ArrayList<>();

        QueryWrapper<EnvMoveCall> callQueryWrapper = new QueryWrapper<>();
        callQueryWrapper.eq("farm_id", farmId).eq("duck_num", duckNum);
        List<EnvMoveCall> moveCalls = moveCallMapper.selectList(callQueryWrapper);
        if (moveCalls.size() == 0) {
            QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("duck_num", duckNum);
            BaseDuckInfo baseDuckInfo = duckInfoMapper.selectOne(queryWrapper);
            int i = 7;
            QueryWrapper<EnvMoveCall> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("farm_id", farmId);
            List<EnvMoveCallVo> moveCallVoList = moveCallMapper.listScreen2(queryWrapper1);
            BigDecimal avgWeight = BigDecimal.ZERO;
            if (moveCallVoList.size() > 6) {
                for (int j = 0; j < 7; j++) {
                    EnvMoveCallVo moveCallVo = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -i);
                    moveCallVo.setCallDate(dateTime1);
                    moveCallVo.setDuckWeight(baseDuckInfo.getNowWeight());
                    if (moveCallVoList.size() > 7) {
                        moveCallVo.setAvgWeight(moveCallVoList.get(moveCallVoList.size() - 7 + j).getAvgWeight());
                    } else {
                        if (ObjectUtil.isNotEmpty(moveCallVoList.get(j))) {
                            avgWeight = moveCallVoList.get(j).getAvgWeight();
                        }
                        moveCallVo.setAvgWeight(avgWeight);
                    }
                    envMoveCallVoList.add(moveCallVo);
                    i--;
                }
            } else {
                BigDecimal decimal = BigDecimal.ZERO;
                for (EnvMoveCallVo moveCallVo : moveCallVoList) {
                    EnvMoveCallVo moveCallVo1 = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -i);
                    moveCallVo1.setCallDate(dateTime1);
                    moveCallVo1.setDuckWeight(baseDuckInfo.getNowWeight());
                    decimal = moveCallVo.getAvgWeight();
                    moveCallVo1.setAvgWeight(decimal);
                    envMoveCallVoList.add(moveCallVo1);
                    i--;
                }
                int a = 7 - moveCallVoList.size();
                for (int j = 0; j < 7 - moveCallVoList.size(); j++) {
                    EnvMoveCallVo moveCallVo1 = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -a);
                    moveCallVo1.setCallDate(dateTime1);
                    moveCallVo1.setDuckWeight(baseDuckInfo.getNowWeight());
                    moveCallVo1.setAvgWeight(decimal);
                    envMoveCallVoList.add(moveCallVo1);
                    a--;
                }
            }

        jsonObject.put("weight", envMoveCallVoList);
    } else

    {
        QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("call_date", dateTime);
        List<EnvMoveCallVo> envMoveCallVos = moveCallMapper.listScreen2(queryWrapper);
        int i = 7;
        QueryWrapper<EnvMoveCall> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).eq("duck_num", duckNum)
                .orderByDesc("id").last(" limit 1");
        EnvMoveCall call = moveCallMapper.selectOne(queryWrapper2);
        if (envMoveCallVos.size() == 0) {
            QueryWrapper<EnvMoveCall> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("farm_id", farmId);
            List<EnvMoveCallVo> moveCallVoList = moveCallMapper.listScreen2(queryWrapper1);
            BigDecimal avgWeight = BigDecimal.ZERO;
            if (moveCallVoList.size() > 6) {
                for (int j = 0; j < 7; j++) {
                    EnvMoveCallVo moveCallVo = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -i);
                    moveCallVo.setCallDate(dateTime1);
                    moveCallVo.setDuckWeight(call.getDuckWeight());
                    if (moveCallVoList.size() > 7) {
                        moveCallVo.setAvgWeight(moveCallVoList.get(moveCallVoList.size() - 7 + j).getAvgWeight());
                    } else {
                        if (ObjectUtil.isNotEmpty(moveCallVoList.get(j))) {
                            avgWeight = moveCallVoList.get(j).getAvgWeight();
                        }
                        moveCallVo.setAvgWeight(avgWeight);
                    }
                    envMoveCallVoList.add(moveCallVo);
                    i--;
                }
            } else {
                BigDecimal decimal = BigDecimal.ZERO;
                for (EnvMoveCallVo moveCallVo : moveCallVoList) {
                    EnvMoveCallVo moveCallVo1 = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -i);
                    moveCallVo1.setCallDate(dateTime1);
                    moveCallVo1.setDuckWeight(call.getDuckWeight());
                    decimal = moveCallVo.getAvgWeight();
                    moveCallVo1.setAvgWeight(decimal);
                    envMoveCallVoList.add(moveCallVo1);
                    i--;
                }
                int a = 7 - moveCallVoList.size();
                for (int j = 0; j < 7 - moveCallVoList.size(); j++) {
                    EnvMoveCallVo moveCallVo1 = new EnvMoveCallVo();
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -a);
                    moveCallVo1.setCallDate(dateTime1);
                    moveCallVo1.setDuckWeight(call.getDuckWeight());
                    moveCallVo1.setAvgWeight(decimal);
                    envMoveCallVoList.add(moveCallVo1);
                    a--;
                }
            }


        } else {
            for (int j = 0; j < 7; j++) {
                if (envMoveCallVos.size() > j) {
                    System.out.println("这个是平均值的时间：" + envMoveCallVos.get(j).getCallDate());
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    String format = sdf.format(envMoveCallVos.get(j).getCallDate());
                    System.out.println("格式化后的时间：" + format);
                    EnvMoveCallVo moveCallVo = new EnvMoveCallVo();
                    moveCallVo.setCallDate(sdf.parse(format));
                    String start = format + " 00:00:00";
                    String end = format + " 23:59:59";
                    QueryWrapper<EnvMoveCall> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("farm_id", farmId).eq("duck_num", duckNum)
                            .between("call_date", start, end)
                            .orderByDesc("id")
                            .last(" limit 1");
                    EnvMoveCall envMoveCall = moveCallMapper.selectOne(queryWrapper1);

                    if (ObjectUtil.isEmpty(envMoveCall)) {
                        moveCallVo.setDuckWeight(call.getDuckWeight());
                    } else {
                        moveCallVo.setDuckWeight(envMoveCall.getDuckWeight());
                    }

                    moveCallVo.setAvgWeight(envMoveCallVos.get(j).getAvgWeight());
                    envMoveCallVoList.add(moveCallVo);
                } else {
                    DateTime dateTime1 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -i);
                    EnvMoveCallVo moveCallVo = new EnvMoveCallVo();
                    moveCallVo.setDuckWeight(call.getDuckWeight());
                    moveCallVo.setCallDate(dateTime1);
                    if (ObjectUtil.isNull(envMoveCallVos) || envMoveCallVos.size() < 1) {
                        moveCallVo.setAvgWeight(new BigDecimal(0));
                    } else {
                        moveCallVo.setAvgWeight(envMoveCallVos.get(0).getAvgWeight());
                    }

                    envMoveCallVoList.add(moveCallVo);
                }
                i--;
            }
        }


//            for (EnvMoveCallVo envMoveCallVo : envMoveCallVos) {
//
//                System.out.println("这个是平均值的时间："+envMoveCallVo.getCallDate());
//                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//                String format = sdf.format(envMoveCallVo.getCallDate());
//                System.out.println("格式化后的时间："+format);
//                EnvMoveCallVo moveCallVo = new EnvMoveCallVo();
//                moveCallVo.setCallDate(sdf.parse(format));
//
//                String start= format + " 00:00:00";
//                String end= format + " 23:59:59";
//
//                QueryWrapper<EnvMoveCall> queryWrapper1 = new QueryWrapper<>();
//                queryWrapper1.eq("farm_id", farmId).eq("duck_num", duckNum)
//                        .between("call_date",start,end)
//                        .orderByDesc("id")
//                        .last(" limit 1");
//                EnvMoveCall envMoveCall = moveCallMapper.selectOne(queryWrapper1);
//
//                if (ObjectUtil.isEmpty(envMoveCall)) {
//                    moveCallVo.setDuckWeight(call.getDuckWeight());
//                } else {
//                    moveCallVo.setDuckWeight(envMoveCall.getDuckWeight());
//                }
//
//                moveCallVo.setAvgWeight(envMoveCallVo.getAvgWeight());
//
//                envMoveCallVoList.add(moveCallVo);
//            }
        jsonObject.put("weight", envMoveCallVoList);
    }

    List<EnvRegularCallFeedingVo> feedingVoList = new ArrayList<>();
    QueryWrapper<EnvRegularCallFeeding> feedingQueryWrapper = new QueryWrapper<>();
        feedingQueryWrapper.eq("farm_id",farmId).

    eq("duck_num",duckNum);

    List<EnvRegularCallFeeding> feedings = feedingMapper.selectList(feedingQueryWrapper);
        if(feedings.size()==0)

    {
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("event_type", 0).ge("call_date", dateTime);
        List<EnvRegularCallFeedingVo> envRegularCallFeedingVos = feedingMapper.listScreen2(queryWrapper1);
        queryWrapper1.groupBy("duck_num");
        List<EnvRegularCallFeeding> list = feedingMapper.selectList(queryWrapper1);
        for (EnvRegularCallFeedingVo envRegularCallFeedingVo : envRegularCallFeedingVos) {
            EnvRegularCallFeedingVo feedingVo = new EnvRegularCallFeedingVo();
            BigDecimal value = BigDecimal.valueOf(envRegularCallFeedingVo.getDuckWeight());
            BigDecimal decimal = BigDecimal.valueOf(list.size());
            BigDecimal decimal1 = value.divide(decimal, 2);//pingjun
            feedingVo.setAvgWeight(decimal1);
            feedingVo.setDuckWeight(0);
            feedingVo.setCallDate(envRegularCallFeedingVo.getCallDate());
            feedingVoList.add(feedingVo);
        }
        jsonObject.put("feed", feedingVoList);
    } else

    {
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("event_type", 0);
        queryWrapper1.groupBy("duck_num");
        List<EnvRegularCallFeeding> list = feedingMapper.selectList(queryWrapper1);
        QueryWrapper<EnvRegularCallFeeding> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).eq("duck_num", duckNum).eq("event_type", 0).ge("call_date", dateTime);
        List<EnvRegularCallFeedingVo> feedingVoList1 = feedingMapper.listScreen2(queryWrapper2);//某只鸭子的总采食量
        int i = 0;
        for (EnvRegularCallFeedingVo envRegularCallFeedingVo : feedingVoList1) {
            Date callDate = envRegularCallFeedingVo.getCallDate();
            System.out.println("这个是平均值的时间：" + callDate);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String format = sdf.format(callDate);
            System.out.println("格式化后的时间：" + format);
            String start = format + " 00:00:00";
            String end = format + " 23:59:59";
            QueryWrapper<EnvRegularCallFeeding> callFeedingQueryWrapper = new QueryWrapper<>();
            callFeedingQueryWrapper.eq("farm_id", farmId).eq("event_type", 0).between("call_date", start, end);
            callFeedingQueryWrapper.select("IFNULL(SUM(duck_weight),0) duckWeight");
            EnvRegularCallFeeding feeding = feedingMapper.selectOne(callFeedingQueryWrapper);
            EnvRegularCallFeedingVo feedingVo = new EnvRegularCallFeedingVo();
            BigDecimal value = BigDecimal.valueOf(feeding.getDuckWeight());
            BigDecimal decimal = BigDecimal.valueOf(list.size());
            BigDecimal decimal1 = value.divide(decimal, 2);//pingjun
            feedingVo.setAvgWeight(decimal1);
            feedingVo.setDuckWeight(envRegularCallFeedingVo.getDuckWeight());
            feedingVo.setCallDate(envRegularCallFeedingVo.getCallDate());
            feedingVoList.add(feedingVo);
            i++;
        }
        jsonObject.put("feed", feedingVoList);
    }
        return new

    Result(ResultCode.SUCCESS, jsonObject);

}

    @Override
    public Result listAppDuck(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String chiNum = paramsMap.get("chiNum");
        String jiaoNum = paramsMap.get("jiaoNum");
        String farmId = paramsMap.get("farmId");//场的编码
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(chiNum), "chi_num", chiNum)
                .like(StringUtils.isNotBlank(jiaoNum), "jiao_num", jiaoNum).eq("farm_id", farmId).orderByDesc("id");
        Page<BaseDuckInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, duckInfoMapper.selectPage(page, queryWrapper));
    }


}
