package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.Breed;
import com.huimv.guowei.admin.entity.Departure;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.BreedMapper;
import com.huimv.guowei.admin.service.IBreedService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-14
 */
@Service
public class BreedServiceImpl extends ServiceImpl<BreedMapper, Breed> implements IBreedService {
    @Autowired
    private BreedMapper breedMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<Breed> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, breedMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<Breed> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName),"unit_name", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"male_chi_num", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"female_chi_num", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"male_jiao_num", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"female_jiao_num", unitName)
                .orderByDesc("id");
        Page<Breed> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, breedMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result importData(HttpServletRequest httpServletRequest, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("雄鸭翅号" , "maleChiNum");
        map.put("雌鸭翅号" , "femaleChiNum");
        map.put("配种日期" , "date");
        map.put("雄鸭脚号" , "maleJiaoNum");
        map.put("雌鸭脚号" , "femaleJiaoNum");
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<Breed> list = ExcelImportSheet.getListFromExcel(sheet , Breed.class , map);

            for (Breed breed : list) {
                //底层数据库操作 insert什么的
                QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("chi_num", breed.getFemaleChiNum());
                BaseDuckInfo duckInfo = infoMapper.selectOne(queryWrapper);
                breed.setFemaleNum(duckInfo.getDuckNum());
                breed.setUnitId(duckInfo.getUnitId());
                breed.setUnitName(duckInfo.getUnitName());
                breed.setFarmId(duckInfo.getFarmId());
                QueryWrapper<BaseDuckInfo> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("chi_num", breed.getMaleChiNum());
                BaseDuckInfo duckInfo1 = infoMapper.selectOne(queryWrapper);
                breed.setMaleNum(duckInfo1.getDuckNum());
                breedMapper.insert(breed);
                System.out.println(breed.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        }finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }

    @Override
    public Result listByChiNum(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String chiNum = paramsMap.get("chiNum");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<Breed> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("male_chi_num", chiNum)
                .or()
                .eq("female_chi_num", chiNum)
                .orderByDesc("id");
        Page<Breed> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, breedMapper.selectPage(page, queryWrapper));
    }
}
