package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.Batch;
import com.huimv.guowei.admin.entity.Departure;
import com.huimv.guowei.admin.entity.DuckHealthInfo;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.BatchMapper;
import com.huimv.guowei.admin.mapper.DepartureMapper;
import com.huimv.guowei.admin.service.IDepartureService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-14
 */
@Service
public class DepartureServiceImpl extends ServiceImpl<DepartureMapper, Departure> implements IDepartureService {

    @Autowired
    private DepartureMapper departureMapper;
    @Autowired
    private BatchMapper batchMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        QueryWrapper<Departure> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId)
                .like(StringUtils.isNotBlank(unitName),"unit_name", unitName)
                .orderByDesc("date").last(" limit 10");

        return new Result(ResultCode.SUCCESS, departureMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<Departure> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName),"unit_name", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"chi_num", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"jiao_num", unitName)
                .orderByDesc("id");
        Page<Departure> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, departureMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listBatch(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        return null;
    }

    @Override
    public Result editBatch(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String batchNum = paramsMap.get("batchNum");
        String reason = paramsMap.get("reason");
        String date = paramsMap.get("date");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("batch_num", batchNum);
        List<BaseDuckInfo> baseDuckInfos = infoMapper.selectList(queryWrapper);
        for (BaseDuckInfo baseDuckInfo : baseDuckInfos) {
            if (baseDuckInfo.getIsCage() != 1) {
                baseDuckInfo.setIsCage(1);
                Departure departure = new Departure();
                departure.setBatchNum(baseDuckInfo.getBatchNum());
                departure.setJiaoNum(baseDuckInfo.getJiaoNum());
                departure.setChiNum(baseDuckInfo.getChiNum());
                departure.setDate(sdf.parse(date));
                departure.setFarmId(baseDuckInfo.getFarmId());
                departure.setUnitName(baseDuckInfo.getUnitName());
                departure.setUnitId(baseDuckInfo.getUnitId());
                departure.setDuckNum(baseDuckInfo.getDuckNum());
                departure.setReason(reason);
                departure.setType(2);
                infoMapper.updateById(baseDuckInfo);
                departureMapper.insert(departure);
            }
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listByChiNum(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String chiNum = paramsMap.get("chiNum");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<Departure> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("chi_num", chiNum)
                .orderByDesc("id");
        Page<Departure> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, departureMapper.selectPage(page, queryWrapper));
    }
}
