package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.Breed;
import com.huimv.guowei.admin.entity.DuckBodySizeInfo;
import com.huimv.guowei.admin.entity.vo.DuckBodySizeInfoVo;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.DuckBodySizeInfoMapper;
import com.huimv.guowei.admin.service.IDuckBodySizeInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-14
 */
@Service
public class DuckBodySizeInfoServiceImpl extends ServiceImpl<DuckBodySizeInfoMapper, DuckBodySizeInfo> implements IDuckBodySizeInfoService {
    @Resource
    private DuckBodySizeInfoMapper duckBodySizeInfoMapper;
    @Resource
    private BaseDuckInfoMapper baseDuckInfoMapper;

    @Override
    public Result listLimitInfo(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String num = paramsMap.get("num");
        List<DuckBodySizeInfoVo> duckBodySizeInfoList = duckBodySizeInfoMapper.listLimitInfo(num,farmId);
        return new Result(ResultCode.SUCCESS,duckBodySizeInfoList);
    }

    @Override
    public Result listPageInfo(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNo = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String chiNum = paramsMap.get("chiNum");
        String unitName = paramsMap.get("unitName");
        Page<DuckBodySizeInfoVo> page = new Page(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<DuckBodySizeInfoVo> duckBodySizeInfoPage = duckBodySizeInfoMapper.listPageInfo(page,chiNum,unitName,farmId);
        return new Result(ResultCode.SUCCESS,duckBodySizeInfoPage);
    }

    @Override
    public Result getLastInfo(Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        DuckBodySizeInfo duckBodySizeInfo = duckBodySizeInfoMapper.getLastInfo(baseDuckInfoMapper.selectById(id).getDuckNum());
        if (ObjectUtil.isEmpty(duckBodySizeInfo)){
            duckBodySizeInfo = new DuckBodySizeInfo();
            duckBodySizeInfo.setBodyLength("");
            duckBodySizeInfo.setChestDeep("");
            duckBodySizeInfo.setChestWide("");
            duckBodySizeInfo.setHalfWaterLength("");
            duckBodySizeInfo.setKeelLength("");
            duckBodySizeInfo.setPelvicWide("");
            duckBodySizeInfo.setShinPerimeter("");
            duckBodySizeInfo.setShinWide("");
        }
        return new Result(ResultCode.SUCCESS,duckBodySizeInfo);
    }

    @Override
    public Result importData(HttpServletRequest httpServletRequest, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("翅号" , "chiNum");
        map.put("脚号" , "jiaoNum");
        map.put("代次号" , "batchNum");
        map.put("胫围" , "shinPerimeter");
        map.put("胫长" , "shinWide");
        map.put("骨盆宽" , "pelvicWide");
        map.put("胸深" , "chestDeep");
        map.put("胸宽" , "chestWide");
        map.put("半潜水长" , "halfWaterLength");
        map.put("龙骨长" , "keelLength");
        map.put("体斜长" , "bodyLength");
        map.put("测量日期" , "measureDate");
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<DuckBodySizeInfo> list = ExcelImportSheet.getListFromExcel(sheet , DuckBodySizeInfo.class , map);

            for (DuckBodySizeInfo breed : list) {
                //底层数据库操作 insert什么的
                QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("chi_num", breed.getChiNum());
                BaseDuckInfo baseDuckInfo = baseDuckInfoMapper.selectOne(queryWrapper);
                breed.setFarmId(baseDuckInfo.getFarmId());
                breed.setDuckNum(baseDuckInfo.getDuckNum());
                breed.setDuckId(baseDuckInfo.getId());
                breed.setUnitId(baseDuckInfo.getUnitId());
                breed.setUnitName(baseDuckInfo.getUnitName());
                duckBodySizeInfoMapper.insert(breed);
                System.out.println(breed.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        }finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }
}
