package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.entity.vo.BaseDuckInfoImportData;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.DuckEggTextureInfoMapper;
import com.huimv.guowei.admin.service.IDuckEggTextureInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-14
 */
@Service
public class DuckEggTextureInfoServiceImpl extends ServiceImpl<DuckEggTextureInfoMapper, DuckEggTextureInfo> implements IDuckEggTextureInfoService {
    @Resource
    private DuckEggTextureInfoMapper duckEggTextureInfoMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;

    @Override
    public Result listLimitInfo(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String num = paramsMap.get("num");
        List<DuckEggTextureInfo> duckEggTextureInfoList = duckEggTextureInfoMapper.selectList(new QueryWrapper<DuckEggTextureInfo>()
                .eq("farm_id",farmId).orderByDesc("id").last("limit "+num));
        return new Result(ResultCode.SUCCESS,duckEggTextureInfoList);
    }

    @Override
    public Result listPageInfo(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNo = paramsMap.get("pageNum");
        String chiNum = paramsMap.get("chiNum");
        String pageSize = paramsMap.get("pageSize");
        String unitName = paramsMap.get("unitName");
        Page<DuckEggTextureInfo> page = new Page(Integer.parseInt(pageNo),Integer.parseInt(pageSize));
        Page<DuckEggTextureInfo> duckEggTextureInfoPage = duckEggTextureInfoMapper.selectPage(page,
                new QueryWrapper<DuckEggTextureInfo>().like(!StringUtils.isEmpty(chiNum),"chi_num",chiNum).like("unit_name", unitName)
                        .or(wrapper -> wrapper.like("chi_num", unitName).or().like("jiao_num", unitName))
                        .eq("farm_id",farmId).orderByDesc("id"));
        return new Result(ResultCode.SUCCESS,duckEggTextureInfoPage);
    }
    @Override
    public Result importData(HttpServletRequest httpServletRequest, Map<String,String> paramsMap, MultipartFile files) {
        String farmId = paramsMap.get("farmId");
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("翅号" , "chiNum");
        map.put("脚号" , "jiaoNum");
        map.put("代次号" , "batchNum");
        map.put("产蛋日期" , "layEggsTime");
        map.put("测定日期" , "determineTime");
        map.put("蛋壳颜色" , "eggColour");
        map.put("钝端厚度" , "bluntThick");
        map.put("锐端厚度" , "sharpThick");
        map.put("侧面厚度" , "sideThick");
        map.put("长径" , "longDiameter");
        map.put("短径" , "shortDiameter");
        map.put("蛋壳强度" , "eggStrength");
        map.put("Wgt" , "wgt");
        map.put("Hgt" , "hgt");
        map.put("Col" , "col");
        map.put("Hu" , "hu");
        map.put("等级" , "grade");
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<DuckEggTextureInfo> list = ExcelImportSheet.getListFromExcel(sheet , DuckEggTextureInfo.class , map);

            for (DuckEggTextureInfo breed : list) {
                //底层数据库操作 insert什么的
                QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", farmId).eq("chi_num", breed.getChiNum());
                BaseDuckInfo duckInfo = infoMapper.selectOne(queryWrapper);
                breed.setDuckNum(duckInfo.getDuckNum());
                breed.setDuckId(duckInfo.getId());
                breed.setUnitId(duckInfo.getUnitId());
                breed.setUnitName(duckInfo.getUnitName());
                breed.setFarmId(duckInfo.getFarmId());
                QueryWrapper<DuckEggTextureInfo> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("farm_id", farmId).orderByDesc("id").last(" limit 1");
                DuckEggTextureInfo textureInfo = duckEggTextureInfoMapper.selectOne(queryWrapper1);
                if (ObjectUtil.isEmpty(textureInfo)) {
                    breed.setEggsNum("100001");
                } else {
                    breed.setEggsNum(String.valueOf(Integer.parseInt(textureInfo.getEggsNum()) + 1));
                }
                if (duckInfo.getDuckSex() == 1) {
                    duckEggTextureInfoMapper.insert(breed);
                }
//                System.out.println(breed.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        }finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }
}
