package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.Breed;
import com.huimv.guowei.admin.entity.DuckHealthInfo;
import com.huimv.guowei.admin.entity.DuckImmunityInfo;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.DuckImmunityInfoMapper;
import com.huimv.guowei.admin.service.IDuckImmunityInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.ExcelImportSheet;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class DuckImmunityInfoServiceImpl extends ServiceImpl<DuckImmunityInfoMapper, DuckImmunityInfo> implements IDuckImmunityInfoService {

    @Autowired
    private DuckImmunityInfoMapper infoMapper;
    @Autowired
    private BaseDuckInfoMapper baseDuckInfoMapper;
    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String duckNum = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");

        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }

        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        if (startDate == null || "".equals(startDate)) {
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).orderByAsc("health_date");
        }else {
            startDate = startDate + " 00:00:00";
            endDate = endDate + " 23:59:59";
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).between("health_date", startDate, endDate).orderByAsc("health_date");
        }
        Page<DuckImmunityInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, infoMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result addImmunity(HttpServletRequest httpServletRequest, DuckImmunityInfo duckImmunityInfo) throws ParseException {

        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("duck_num", duckImmunityInfo.getDuckNum());
        BaseDuckInfo baseDuckInfo = baseDuckInfoMapper.selectOne(queryWrapper);
        duckImmunityInfo.setDuckId(baseDuckInfo.getId());
        duckImmunityInfo.setUnitName(baseDuckInfo.getUnitName());
        duckImmunityInfo.setUnitId(baseDuckInfo.getUnitId());
        duckImmunityInfo.setDuckNum(baseDuckInfo.getDuckNum());
        duckImmunityInfo.setChiNum(baseDuckInfo.getChiNum());
        duckImmunityInfo.setJiaoNum(baseDuckInfo.getJiaoNum());
        duckImmunityInfo.setBatchNum(baseDuckInfo.getBatchNum());
        infoMapper.insert(duckImmunityInfo);
        return new Result(ResultCode.SUCCESS);
    }
    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("health_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, infoMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName),"unit_name", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"chi_num", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName),"jiao_num", unitName)
        .orderByDesc("health_date");
        Page<DuckImmunityInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, infoMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result importData(HttpServletRequest httpServletRequest, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("接种疫苗" , "vaccination");
        map.put("翅号" , "chiNum");
        map.put("注射部位" , "injection");
        map.put("剂量" , "dose");
        map.put("代次号" , "batchNum");
        map.put("脚号" , "jiaoNum");
        map.put("接种日期" , "healthDate");
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<DuckImmunityInfo> list = ExcelImportSheet.getListFromExcel(sheet , DuckImmunityInfo.class , map);

            for (DuckImmunityInfo breed : list) {
                //底层数据库操作 insert什么的
                QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("chi_num", breed.getChiNum());
                BaseDuckInfo baseDuckInfo = baseDuckInfoMapper.selectOne(queryWrapper);
                breed.setFarmId(baseDuckInfo.getFarmId());
                breed.setUnitId(baseDuckInfo.getUnitId());
                breed.setUnitName(baseDuckInfo.getUnitName());
                breed.setDuckNum(baseDuckInfo.getDuckNum());
                breed.setDuckId(baseDuckInfo.getId());
                infoMapper.insert(breed);
                System.out.println(breed.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        }finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }

    @Override
    public Result listByChiNum(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String chiNum = paramsMap.get("chiNum");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("chi_num", chiNum)
                .orderByDesc("health_date");
        Page<DuckImmunityInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, infoMapper.selectPage(page, queryWrapper));
    }

}
