package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.EnergyElectricity;
import com.huimv.guowei.admin.entity.EnergyWater;
import com.huimv.guowei.admin.entity.vo.EnergyWaterVo;
import com.huimv.guowei.admin.mapper.BaseBuildingMapper;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.EnergyElectricityMapper;
import com.huimv.guowei.admin.service.IEnergyElectricityService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.Print;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.xml.crypto.Data;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnergyElectricityServiceImpl extends ServiceImpl<EnergyElectricityMapper, EnergyElectricity> implements IEnergyElectricityService {
    @Resource
    private EnergyElectricityMapper energyElectricityMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;


    @Override
    public Result getElectricity(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Map resultMap = new HashMap();
        BigDecimal dayElectricity = energyElectricityMapper.getDayElectricity(farmId);
        if (dayElectricity == null){
            dayElectricity = BigDecimal.ZERO;
        }
        BigDecimal monthElectricity = energyElectricityMapper.getMonthElectricity(farmId);
        if (monthElectricity == null){
            monthElectricity = BigDecimal.ZERO;
        }
        resultMap.put("dayElectricity",dayElectricity);
        resultMap.put("MonthElectricity",monthElectricity);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listElectricity(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";

        List<EnergyElectricity> energyElectricityList = null;
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listDayDetail(farmId);
        }
        if ("2".equals(type)){
            energyElectricityList = energyElectricityMapper.listWeekDetail(farmId);
        }
        if ("3".equals(type)){
            energyElectricityList = energyElectricityMapper.listMonthDetail(farmId);
        }
        if ("4".equals(type)){
            energyElectricityList = energyElectricityMapper.listCustomDetail(farmId,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }

    @Override
    public Result getScreenElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        Date timesMonthmorning = DataUill.getTimesMonthmorning();
        BigDecimal dayGap,monthGap;
        String format = "";
        String format1 = "";
        Map resultMap = new HashMap();
        //今日
        QueryWrapper<EnergyElectricity> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesmorning);
        BigDecimal dayElectricity = energyElectricityMapper.getElectricity(queryWrapper1);
        //昨日
        LocalDate yesterday = LocalDate.now().minusDays(1);
        LocalDateTime startOfDay = yesterday.atStartOfDay();
        LocalDateTime endOfDay = yesterday.atTime(LocalTime.MAX);
        QueryWrapper<EnergyElectricity> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",startOfDay,endOfDay);
        BigDecimal lastDayElectricity = energyElectricityMapper.getElectricity(queryWrapper2);
        //日环比
        if (lastDayElectricity.compareTo(BigDecimal.ZERO)==0){
            format = "100";
        }else {
            dayGap = dayElectricity.subtract(lastDayElectricity).divide(lastDayElectricity,2,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100));
            DecimalFormat def = new DecimalFormat("0.00");
            format = def.format(dayGap);
        }

        //本月
        QueryWrapper<EnergyElectricity> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesMonthmorning);
        BigDecimal monthElectricity = energyElectricityMapper.getElectricity(queryWrapper3);
        //上月
        Date lastMonthStartMorning = DataUill.getLastMonthStartMorning();
        Date lastMonthEndNight = DataUill.getLastMonthEndNight();
        QueryWrapper<EnergyElectricity> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",lastMonthStartMorning,lastMonthEndNight);
        BigDecimal lastMonthElectricity = energyElectricityMapper.getElectricity(queryWrapper4);
        //月环比
        System.out.println(monthElectricity);
        System.out.println(lastMonthElectricity);
        if (lastMonthElectricity.compareTo(BigDecimal.ZERO)==0){
            format1 = "100";
        }else {
            monthGap = monthElectricity.subtract(lastMonthElectricity).divide(lastMonthElectricity,2,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100));
            DecimalFormat def = new DecimalFormat("0.00");
            format1 = def.format(monthGap);
        }
        int dayTrend =dayElectricity.compareTo(lastDayElectricity);
        int monthTrend =monthElectricity.compareTo(lastMonthElectricity);
        resultMap.put("dayElectricity",dayElectricity);
        resultMap.put("dayTrend",dayTrend);
        resultMap.put("dayElectricityGap",format+"%");
        resultMap.put("monthElectricity",monthElectricity);
        resultMap.put("monthTrend",monthTrend);
        resultMap.put("monthElectricityGap",format1+"%");
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listScreenElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        List<EnergyElectricity> energyElectricityList;
        //六个月
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listSixMonthElectricity(farmId, unitId);
        }
        //七天
        else {
            energyElectricityList = energyElectricityMapper.listSevenDayElectricity(farmId, unitId);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }

    @Override
    public Result listUnitElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        List<EnergyElectricity> energyElectricityList;
        //日
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listUnitDayElectricity(farmId, unitId);
        }
        //月
        else {
            energyElectricityList = energyElectricityMapper.listUnitMonthElectricity(farmId, unitId);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }

    @Override
    public void printElectricity(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");//选择要导出的类型
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");

        List<EnergyWaterVo> waterVos = new ArrayList<>();
        //今日
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        if (type.equals("1")) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("create_date", timesmorning);
            waterVos = energyElectricityMapper.printElectricity(queryWrapper);
        }
        //本周

        if (type.equals("2")) {
            DateTime dateTime = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("create_date", dateTime);
            waterVos = energyElectricityMapper.printElectricity(queryWrapper);
        }
        //本月

        if (type.equals("3")) {
            Date timesMonthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("create_date", timesMonthmorning);
            waterVos = energyElectricityMapper.printElectricity(queryWrapper);
        }
        //自定义

        if (type.equals("4")) {
            endTime = endTime + " 23:59:59";
            queryWrapper.between("create_date", startTime, endTime);
            waterVos = energyElectricityMapper.printElectricity(queryWrapper);
        }
        List<EnergyWaterVo> waterVoList = new ArrayList<>();
        for (EnergyWaterVo waterVo : waterVos) {
            EnergyWaterVo waterVo1 = new EnergyWaterVo();
            BaseBuilding baseBuilding = buildingMapper.selectById(waterVo.getChipName());
            waterVo1.setWaterValue(waterVo.getWaterValue());
            if (ObjectUtil.isNotEmpty(baseBuilding)) {
                waterVo1.setChipName(baseBuilding.getBuildName());
            }
            waterVo1.setCreateDate(waterVo.getCreateDate());
            waterVoList.add(waterVo1);
        }
        Print.printElectricity(waterVoList);
    }
}
