package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.EnvData;
import com.huimv.guowei.admin.entity.EnvDevice;
import com.huimv.guowei.admin.entity.EnvDeviceEquipment;
import com.huimv.guowei.admin.entity.vo.EnvDataGetOneVo;
import com.huimv.guowei.admin.entity.vo.EnvDataVo;
import com.huimv.guowei.admin.mapper.BaseBuildingMapper;
import com.huimv.guowei.admin.mapper.EnvDataMapper;
import com.huimv.guowei.admin.mapper.EnvDeviceEquipmentMapper;
import com.huimv.guowei.admin.mapper.EnvDeviceMapper;
import com.huimv.guowei.admin.service.IEnvDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvDataServiceImpl extends ServiceImpl<EnvDataMapper, EnvData> implements IEnvDataService {

    @Resource
    private EnvDataMapper envDataMapper;

    @Resource
    private BaseBuildingMapper baseBuildingMapper;
    @Resource
    private EnvDeviceMapper deviceMapper;
    @Autowired
    private EnvDeviceEquipmentMapper equipmentMapper;

    @Override
    public Result getLastFlow(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<EnvDataVo> envDataList = envDataMapper.getLastFlow(farmId);
        return new Result(ResultCode.SUCCESS,envDataList);
    }

    @Override
    public Result listByBuild(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";
        String type = paramsMap.get("type");

        List<EnvData> envDataList = null;
//      默认情况:第一个栋舍的本周数据
        if (StringUtils.isBlank(type)){
            List<BaseBuilding> baseBuildings = baseBuildingMapper.selectList(new QueryWrapper<BaseBuilding>().eq("parent_id", 0));
            if (ObjectUtil.isEmpty(baseBuildings)){
                return new Result(10001,"请先添加栋舍!",false);
            }
            unitId = String.valueOf(baseBuildings.get(0).getId());
            envDataList = envDataMapper.listWeekDetail(farmId,unitId);
        }

        //今日查询
        if ("1".equals(type)){
            envDataList = envDataMapper.listDayDetail(farmId,unitId);
        }
        //本周查询
        if ("2".equals(type)){
            envDataList = envDataMapper.listWeekDetail(farmId,unitId);
        }
        //本月查询
        if ("3".equals(type)){
            envDataList = envDataMapper.listMonthDetail(farmId,unitId);
        }
        //自定义查询
        if ("4".equals(type)){
            envDataList = envDataMapper.listCustomDetail(farmId,unitId,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,envDataList);
    }

    @Override
    public Result getOneData(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnvData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_time")
                .last(" limit 1");
        EnvData envData = envDataMapper.selectOne(queryWrapper);
        QueryWrapper<EnvDevice> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("device_code", envData.getDeviceId());
        EnvDevice device = deviceMapper.selectOne(queryWrapper1);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("deviceCode", envData.getDeviceId());
        jsonObject.put("createTime", envData.getCreateTime());
        jsonObject.put("envHum", envData.getEnvHum());
        jsonObject.put("envTemp", envData.getEnvTemp());
        jsonObject.put("unitName", device.getUnitName());
        jsonObject.put("unitId", device.getUnitId());
/*        JSONArray jsonArray = new JSONArray();
        List<EnvDataGetOneVo> data = envDataMapper.getOneData(farmId);
        for (EnvDataGetOneVo datum : data) {
            QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("device_code", datum.getDeviceCode());
            EnvDevice device = deviceMapper.selectOne(queryWrapper);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("deviceCode", datum.getDeviceCode());
            jsonObject.put("createTime", datum.getCreateTime());
            jsonObject.put("envHum", datum.getEnvHum());
            jsonObject.put("envTemp", datum.getEnvTemp());
            jsonObject.put("unitName", device.getUnitName());
            jsonObject.put("unitId", device.getUnitId());
            jsonArray.add(jsonObject);
        }*/
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        Date timesmorning = DataUill.getTimesmorning();
        QueryWrapper<EnvData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId)
                .ge("create_time", timesmorning)
                .orderByAsc("create_time");
        List<EnvData> dataList = envDataMapper.selectList(queryWrapper);
        return new Result(ResultCode.SUCCESS, dataList);
    }

    @Override
    public Result listTodayByBuild(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        List<EnvData> envDataList = null;
        if (unitId.equals("95")){
            envDataList = envDataMapper.listDetail(farmId,unitId);
            Collections.reverse(envDataList);
        }else {
            envDataList = envDataMapper.listDayDetail(farmId,unitId);
        }
        return new Result(ResultCode.SUCCESS,envDataList);
    }

    @Override
    public Result screenEnv(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnvData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("id").last(" limit 1");
        EnvData envData = envDataMapper.selectOne(queryWrapper);
        JSONObject jsonObject = new JSONObject();
        if (ObjectUtil.isNotEmpty(envData)) {
            jsonObject.put("temp", envData.getEnvTemp());
            jsonObject.put("hum", envData.getEnvHum());
        } else {
            jsonObject.put("temp", 0);
            jsonObject.put("hum", 0);
        }
        if ("9942".equals(unitId)) {
            List<EnvDeviceEquipment> list = equipmentMapper.selectList(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId));
            jsonObject.put("light1", list.get(0).getUploadStart());
            jsonObject.put("light2", list.get(1).getUploadStart());
        }
        if ("9943".equals(unitId)) {
            List<EnvDeviceEquipment> list = equipmentMapper.selectList(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id", unitId));
            jsonObject.put("light1", list.get(0).getUploadStart());
            jsonObject.put("light2", list.get(1).getUploadStart());
        }
        jsonObject.put("nh3", 0);
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

}
