package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.entity.vo.EnvRegularCallEggVo;
import com.huimv.guowei.admin.entity.vo.FeedingAndEggVo;
import com.huimv.guowei.admin.mapper.*;
import com.huimv.guowei.admin.service.IEnvRegularCallEggService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.Print;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 固定电子秤 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvRegularCallEggServiceImpl extends ServiceImpl<EnvRegularCallEggMapper, EnvRegularCallEgg> implements IEnvRegularCallEggService {

    @Autowired
    private EnvRegularCallEggMapper eggMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;
    @Autowired
    private EnvRegularCallFeedingMapper feedingMapper;
    @Autowired
    private EnvDataMapper dataMapper;
    @Autowired
    private EnergyElectricityMapper electricityMapper;
    @Autowired
    private EnergyWaterMapper waterMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;
    @Autowired
    private EnvDeviceMapper deviceMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//显示类型
        String duckCode = paramsMap.get("id");//鸭子的id
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        if ("1".equals(type)) {
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_id", duckCode).eq("farm_id", farmId);
            Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, eggMapper.listPage(page, queryWrapper));
        } else {
            String dateType = paramsMap.get("dateType");
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("duck_id", duckCode);
            if ("1".equals(dateType)) {
                Date timesmorning = DataUill.getTimesmorning();
                queryWrapper.ge("call_date", timesmorning);

            } else if ("2".equals(dateType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                queryWrapper.ge("call_date", calendar.getTime());

            } else if ("3".equals(dateType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                queryWrapper.ge("call_date", timesMonthmorning);

            } else {
                String startDate = paramsMap.get("startDate");
                String endDate = paramsMap.get("endDate");
                startDate = startDate + " 00:00:00";
                endDate = endDate + " 23:59:59";
                queryWrapper.between("call_date", startDate, endDate);
            }
            return new Result(ResultCode.SUCCESS, eggMapper.listDay(queryWrapper));
        }
    }

    @Override
    public Result addEgg(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("duckId");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", duckId);
        BaseDuckInfo baseDuckInfo = infoMapper.selectOne(queryWrapper);
        String date = paramsMap.get("date");
        date = date + " 00:00:00";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String eggNum = paramsMap.get("eggNum");
        if (Integer.parseInt(eggNum) < 0) {
            return new Result(10001, "蛋的个数不能为负数", false);
        }
        String eggWeight = paramsMap.get("eggWeight");
        BigDecimal weight = new BigDecimal(eggWeight);
        if (weight.compareTo(BigDecimal.ZERO) != 1) {
            return new Result(10001, "蛋的重量必须大于0", false);
        }
        EnvRegularCallEgg egg = new EnvRegularCallEgg();
        egg.setDuckNum(baseDuckInfo.getDuckNum());
        egg.setFarmId(Integer.parseInt(farmId));
        egg.setDuckWeight(Double.valueOf(eggWeight));
        egg.setEggNum(Integer.parseInt(eggNum));
        egg.setCallDate(sdf.parse(date));
        egg.setDuckId(Integer.parseInt(duckId));
        egg.setChiNum(baseDuckInfo.getChiNum());
        egg.setJiaoNum(baseDuckInfo.getJiaoNum());
        egg.setBatchNum(baseDuckInfo.getBatchNum());
        eggMapper.insert(egg);
        return new Result(10000, "添加成功!", true);
    }

    @Override
    public Result updateEgg(HttpServletRequest httpServletRequest, EnvRegularCallEgg egg) {
        eggMapper.updateById(egg);
        return new Result(10000, "修改成功！", true);
    }

    @Override
    public Result deleteEgg(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");//此时这里就是表中的id
        eggMapper.deleteById(id);
        return new Result(10000, "删除成功!", true);
    }

    @Override
    public void printEgg(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String duckCode = paramsMap.get("duckCode");
        List<EnvRegularCallEggVo> envRegularCallEggVos = feedingMapper.listCount(duckCode);
        Print.printEgg(envRegularCallEggVos);
    }

    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId)
                .like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .orderByDesc("call_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, eggMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }

        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .or()
                .like(StringUtils.isNotBlank(unitName), "chi_num", unitName).or()
                .like(StringUtils.isNotBlank(unitName), "jiao_num", unitName)
                .select("IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum,duck_num duckNum,chi_num chiNum,unit_name unitName,duck_id duckId")
                .groupBy("duck_num")
                .orderByDesc("call_date");
        Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, eggMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listSeven(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("id");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank(duckId), "duck_id", duckId);
        QueryWrapper<EnvRegularCallEgg> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq(StringUtils.isNotBlank(duckId), "duck_id", duckId);
        queryWrapper1.select(" IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum,chi_num chiNum");
        EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper1);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("chiNum", egg.getChiNum());
        jsonObject.put("eggNum", egg.getEggNum());
        jsonObject.put("egg", eggMapper.listDay(queryWrapper));
        jsonObject.put("eggWeight", egg.getDuckWeight());
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result getEggNumById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckNum = paramsMap.get("duckNum");
        String date = paramsMap.get("date");
        String date1 = date + " 00:00:00";

        String date2 = date + " 23:59:59";

        QueryWrapper<BaseDuckInfo> infoQueryWrapper = new QueryWrapper<>();
        infoQueryWrapper.eq("farm_id", farmId).eq("duck_num", duckNum);
        BaseDuckInfo duckInfo = infoMapper.selectOne(infoQueryWrapper);
        if (duckInfo.getDuckSex().equals("0")) {
            return new Result(10001, "雄鸭没有产蛋数量！", false);
        }

        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("duck_num", duckNum);
        queryWrapper.select(" IFNULL(SUM(egg_num),0) eggNum");
        queryWrapper.between("call_date", date1, date2);
        EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper);
        return new Result(ResultCode.SUCCESS, eggMapper.selectOne(queryWrapper));
    }

    @Override
    public Result collector(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String duckNum = paramsMap.get("chiNum");//鸭只编号
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");//传入的是某个舍的id
        JSONArray jsonArray = new JSONArray();
        if ("".equals(duckNum) || null == duckNum) {
            QueryWrapper<BaseBuilding> buildingQueryWrapper = new QueryWrapper<>();
            buildingQueryWrapper.eq("farm_id", farmId).eq("parent_id", unitId);
            List<BaseBuilding> baseBuildings = buildingMapper.selectList(buildingQueryWrapper);//某个舍下所有的笼子

            List<Integer> idLists = new ArrayList<>();
            baseBuildings.forEach(item->{
                idLists.add(item.getId());
            });
            List<FeedingAndEggVo> oneFeedAndEgg = eggMapper.getOneFeedAndEgg(farmId, idLists);
            for (FeedingAndEggVo feedingAndEggVo : oneFeedAndEgg) {
                JSONObject jsonObject = new JSONObject();
                if (ObjectUtil.isNotEmpty(feedingAndEggVo)) {
                    jsonObject.put("egg", feedingAndEggVo.getEggNum());
                    jsonObject.put("feed", feedingAndEggVo.getDuckWeight());
                    jsonObject.put("duckNum", feedingAndEggVo.getDuckNum());
                    jsonObject.put("chiNum", feedingAndEggVo.getChiNum());
                    jsonObject.put("location", feedingAndEggVo.getUnitName());
                    jsonArray.add(jsonObject);
                }
            }
        } else {
            QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(duckNum), "chi_num", duckNum).eq("is_cage", 0);
            List<BaseDuckInfo> baseDuckInfos = infoMapper.selectList(queryWrapper);//笼子里的鸭只信息
            if (baseDuckInfos.size() > 0) {
                List<Integer> idLists = new ArrayList<>();
                baseDuckInfos.forEach(item->{
                    idLists.add(item.getId());
                });
                List<FeedingAndEggVo> oneFeedAndEgg = eggMapper.getOneFeedAndEgg(farmId, idLists);
                JSONObject jsonObject = new JSONObject();
                if (oneFeedAndEgg.size() > 0) {
                    for (FeedingAndEggVo feedingAndEggVo : oneFeedAndEgg) {
                        jsonObject.put("egg", feedingAndEggVo.getEggNum());
                        jsonObject.put("feed", feedingAndEggVo.getDuckWeight());
                        jsonObject.put("duckNum", feedingAndEggVo.getDuckNum());
                        jsonObject.put("chiNum", feedingAndEggVo.getChiNum());
                        jsonObject.put("location", feedingAndEggVo.getUnitName());
                        jsonArray.add(jsonObject);
                    }
                }
            }

        }
        return new Result(ResultCode.SUCCESS, jsonArray);
    }

    @Override
    public Result collectorEnv(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_date").last(" limit 1");
        EnergyElectricity energyElectricity = electricityMapper.selectOne(queryWrapper);//电表

        QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_date").last(" limit 1");
        EnergyWater water = waterMapper.selectOne(queryWrapper1);//水表

        QueryWrapper<EnvData> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_time").last(" limit 1");
        EnvData envData = dataMapper.selectOne(queryWrapper2);//温湿度

        JSONObject jsonObject = new JSONObject();
        if (ObjectUtil.isEmpty(energyElectricity)) {
            jsonObject.put("electricity", 0);
        } else {
            jsonObject.put("electricity", energyElectricity.getElectricityValue());
        }
        if (ObjectUtil.isEmpty(water)) {
            jsonObject.put("water", 0);
        } else {
            jsonObject.put("water", water.getWaterValue());
        }
        if (ObjectUtil.isEmpty(envData)) {
            jsonObject.put("hum", 0);
            jsonObject.put("temp", 0);
        } else {
            jsonObject.put("hum", envData.getEnvHum());
            jsonObject.put("temp", envData.getEnvTemp());
        }

        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result listAll(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("id");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank(duckId), "duck_id", duckId);
        Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, eggMapper.listAll(page, queryWrapper));
    }

}
