package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.common.utils.ResultUtil;
import com.huimv.guowei.admin.entity.SysCamera;
import com.huimv.guowei.admin.entity.SysCameraFunction;
import com.huimv.guowei.admin.entity.SysCameraFunctionArea;
import com.huimv.guowei.admin.mapper.SysCameraFunctionAreaMapper;
import com.huimv.guowei.admin.mapper.SysCameraFunctionMapper;
import com.huimv.guowei.admin.mapper.SysCameraMapper;
import com.huimv.guowei.admin.service.ISysCameraFunctionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class SysCameraFunctionServiceImpl extends ServiceImpl<SysCameraFunctionMapper, SysCameraFunction> implements ISysCameraFunctionService {
    @Autowired
    private SysCameraMapper cameraMapper;

    @Autowired
    private SysCameraFunctionAreaMapper cameraFunctionAreaMapper;

    @Override
    public Result cameraList(Map<String, Object> params) {
        Integer current = (Integer) params.get("current");
        Integer size = (Integer) params.get("size");
        String functionaName = (String) params.get("functionaName");
        Integer farmId = (Integer) params.get("farmId");
        IPage<SysCameraFunction> list = page(new Page<>(current, size), new QueryWrapper<SysCameraFunction>().like(StringUtils.isNotBlank(functionaName), "area_name", functionaName).eq("farm_id", farmId));
        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    @Transactional
    public Result removeIds(List<Integer> ids) {
        for (Integer id : ids) {
            SysCameraFunction byId = this.getById(id);
            List<SysCameraFunctionArea> cameraFunctionAreas = cameraFunctionAreaMapper.selectList(new QueryWrapper<SysCameraFunctionArea>().like("function_ids", id));
            for (SysCameraFunctionArea cameraFunctionArea : cameraFunctionAreas) {
                String functionIds = cameraFunctionArea.getFunctionIds();
                functionIds = functionIds.replaceAll(id+",","");
                functionIds = functionIds.replaceAll("1"+id,"");
                cameraFunctionArea.setFunctionIds(functionIds);
                cameraFunctionAreaMapper.updateById(cameraFunctionArea);
            }
            Integer areaId = cameraMapper.selectCount(new QueryWrapper<SysCamera>().eq("function_id", id));
            if (areaId > 0) {
                return new Result(10001, "删除失败," + byId.getFunctionName() + "仍然存在摄像头", false);
            }
        }
        this.removeByIds(ids);
        return new Result(10000, "删除成功", true);
    }

    @Override
    public Result lsitAndCamera(Integer farmId) {
        List endList = new ArrayList();
        List<SysCameraFunctionArea> cameraFunctionAreas = cameraFunctionAreaMapper.selectList(new QueryWrapper<SysCameraFunctionArea>().eq("farm_id", farmId));
        for (SysCameraFunctionArea cameraFunctionArea : cameraFunctionAreas) {
            String functionIds = cameraFunctionArea.getFunctionIds();
            Map functionAreaMap = new HashMap();
            functionAreaMap.put("cameraName", cameraFunctionArea.getFunctionName());
            functionAreaMap.put("id", cameraFunctionArea.getId());
            functionAreaMap.put("functionList", null);
            if (StringUtils.isNotBlank(functionIds)){
                List<SysCameraFunction> cameraFunctions = this.list(new QueryWrapper<SysCameraFunction>().in("id", functionIds.split(",")));
                List functionList = new ArrayList();
                for (SysCameraFunction cameraFunction : cameraFunctions) {
                    Map functionMap = new HashMap<>();
                    functionMap.put("cameraName", cameraFunction.getFunctionName());
                    functionMap.put("id", cameraFunction.getId());
                    functionMap.put("cameraList", cameraMapper.selectList(new QueryWrapper<SysCamera>().eq("function_id", cameraFunction.getId())));
                    functionList.add(functionMap);

                }
                functionAreaMap.put("functionList", functionList);
            }
            endList.add(functionAreaMap);
        }
        return new Result(ResultCode.SUCCESS, endList);
    }

    @Override
    @Transactional
    public Result addCamera(Map<String, String> params) {
        String functionId = params.get("functionId");
        String carmeraId = params.get("carmeraId");
        String[] split = carmeraId.split(",");
        SysCamera camera1 = new SysCamera();
        camera1.setFunctionId(0);
        cameraMapper.update(camera1,new UpdateWrapper<SysCamera>().eq("function_id",functionId));
        System.out.println(functionId);
        for (String s : split) {
            System.out.println(s);
            SysCamera camera = new SysCamera();
            camera.setId(Integer.parseInt(s));
            camera.setFunctionId(Integer.parseInt(functionId));
            cameraMapper.updateById(camera);
        }

        return new Result(10000, "修改成功", true);
    }

    @Override
    public Result deleteCamera(Map<String, String> params) {
        String carmeraId = params.get("carmeraId");
        String[] split = carmeraId.split(",");
        for (String s : split) {
            SysCamera camera = new SysCamera();
            camera.setFunctionId(0);
            cameraMapper.updateById(camera);
        }
        return ResultUtil.deleteResult(1);
    }

    @Override
    public Result listCamera(Map<String, String> params) {
        String functionId = params.get("functionId");
        List<SysCamera> function_id = cameraMapper.selectList(new QueryWrapper<SysCamera>().eq("function_id", functionId));
        List list = new ArrayList();
        for (SysCamera camera : function_id) {
            list.add(camera.getId());
        }
        return new Result(ResultCode.SUCCESS, list);
    }
}
