package com.huimv.guowei.admin.timer;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.entity.EnvDevice;
import com.huimv.guowei.admin.entity.EnvDeviceOnline;
import com.huimv.guowei.admin.entity.EnvMoveCall;
import com.huimv.guowei.admin.entity.RawData;
import com.huimv.guowei.admin.group.service.IGroupDuckInfoService;
import com.huimv.guowei.admin.service.*;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

@Component
@EnableScheduling
public class DeviceTimer {

    @Resource
    private IEnvDeviceService envDeviceService;
    @Resource
    private IEnvDeviceOnlineService envDeviceOnlineService;
    @Resource
    private IBaseDuckInfoService baseDuckInfoService;

    @Resource
    private IRawDataService rawDataService;
    @Resource
    private IEnvMoveCallService callService;
    @Resource
    private IGroupDuckInfoService groupDuckInfoService;


//    @Scheduled(cron = "*/5 * * * * ?")
    @Scheduled(cron = "0 */5 * * * ?")
    private void saveOnlineDevice(){
        String farmId = "21";
        double rate = 0.0;
        Double deviceOff = (double) envDeviceService.count(new QueryWrapper<EnvDevice>().eq("farm_id", farmId).eq("device_status", 0));
        Double deviceOn = (double) envDeviceService.count(new QueryWrapper<EnvDevice>().eq("farm_id", farmId).eq("device_status", 1));

        Double total = deviceOff+deviceOn;
        try {
            rate = (deviceOn/total)*100;
        }catch (ArithmeticException e){
            System.out.println(e);
        }
        EnvDeviceOnline envDeviceOnline = new EnvDeviceOnline();
        envDeviceOnline.setDeviceOn(deviceOn.intValue());
        envDeviceOnline.setDeviceOff(deviceOff.intValue());
        envDeviceOnline.setOnlineRate(new BigDecimal(rate).setScale(2, RoundingMode.UP));
        envDeviceOnline.setFarmId(farmId);
        envDeviceOnline.setCreatTime(new Date());
        envDeviceOnlineService.save(envDeviceOnline);
    }


//    @Scheduled(cron = "0 * * * * ?")
    @Scheduled(cron = "0 0 23 * * ?")
    private void setDayAge(){
        System.out.println("--->更新日龄");
        baseDuckInfoService.updateDayAge();
        groupDuckInfoService.updateDayAge();
        System.out.println("--->更新成功");
    }


    @Scheduled(cron = "0 0/5 * * * ?")
    private void updateDeviceOn(){
        System.out.println("--->更新固定称在线状态");
        List<EnvDevice> deviceType = envDeviceService.list(new QueryWrapper<EnvDevice>().eq("device_type", 3));
        DateTime dateTime = DateUtil.offsetDay(new Date(), -1);

        for (EnvDevice envDevice : deviceType) {
            String deviceCode = envDevice.getDeviceCode();
            Integer deviceCodeStatus = envDevice.getDeviceStatus();
            int count = rawDataService.count(new QueryWrapper<RawData>().eq("device_code", deviceCode).ge("create_time", dateTime));
            Integer status = count > 0 ? 1 :0 ;
            if (status.equals(deviceCodeStatus)){
                continue;
            }
            envDevice.setDeviceStatus(status);
            envDeviceService.updateById(envDevice);
        }
        System.out.println("--->更新固定称在线状态结束");
    }

    @Scheduled(cron = "0 */1 * * * ?")
    private void updateDevice(){
        List<EnvDevice> envDevices = envDeviceService.list(new QueryWrapper<EnvDevice>().eq("device_type", 2));
        int minutes = 2;
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime time = now.minusMinutes(minutes);
        DateTimeFormatter def = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String format = time.format(def);
        for (EnvDevice envDevice : envDevices) {
            QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("call_code", envDevice.getDeviceCode()).ge("call_date", format);
            int count = callService.count(queryWrapper);
            if (count > 0) {
                envDevice.setDeviceStatus(1);
            } else {
                envDevice.setDeviceStatus(0);
            }
            envDeviceService.updateById(envDevice);
        }
    }
}
