package com.huimv.guowei.admin.timer;

public class Filter {
    /*
     * 重量单位：0.1g
     * Threshold: 当重量波动超过该值则认为是噪声；
     * SteadyCount: 发现噪声后，线性检测下一组稳定重量，当连续SteadyCount个稳定值后，才认为稳定；
     * Accept：稳定评判标准，相临差值不超过Accept则为稳定。
     * */
    private final int Threshold, SteadyCount, Accept;
    private final int[] Data;

    public Filter(int[] data) {
        Data = data;
        Threshold = 10000;
        SteadyCount = 20;
        Accept = 200;
        run();
    }

    public Filter(int[] data, int threshold, int count, int accept) {
        Data = data;
        Threshold = threshold;
        SteadyCount = count;
        Accept = accept;
        run();
    }
    
    public int[] getData() {
        return Data;
    }

    public int calcWeightLoss() {
        int pos1 = 0, weight = 0, pos2;
        while (true) {
            pos2 = nextSharp(pos1);
            if (pos2 == -1) {
                pos2 = Data.length - 1;
                System.out.printf("data[%d]=%d, data[%d]=%d, gap=%d\n", pos1, Data[pos1], pos2, Data[pos2], Data[pos1] - Data[pos2]);
                weight += Data[pos1] - Data[pos2];
                return weight;
            }
            System.out.printf("data[%d]=%d, data[%d]=%d, gap=%d\n", pos1, Data[pos1], pos2, Data[pos2], Data[pos1] - Data[pos2]);
            weight += Data[pos1] - Data[pos2];
            pos1 = pos2 + 1;
        }
    }

    private void run() {
        int pos1 = 0, pos2;
        while (true) {
            pos2 = nextSharp(pos1);
            if (pos2 == -1) {
                update(pos1, Data.length - 1);
                break;
            } else {
                update(pos1, pos2);
                pos1 = pos2 + 1;
            }
        }
    }

    public void show(int start, int count) {
        if (Data == null) {
            System.out.println("[]");
            return;
        }
        System.out.print('[');
        while (count-- > 0) {
            System.out.print(Data[start++]);
            System.out.print(", ");
        }
        System.out.println("\b\b]");
    }

    public void show(int[] data, int start, int count) {
        System.out.print('[');
        while (count-- > 0) {
            System.out.print(data[start++]);
            System.out.print(", ");
        }
        System.out.println("\b\b]");
    }

    private int nextSharp(int start) {
        if (Data == null) return -1;
        while (start < Data.length - 1) {
            if (Data[start + 1] - Data[start] > Threshold) {
                return start;
            }
            start++;
        }
        return -1;
    }

    private void update(int start, int end) {
        if (Data[end] > Data[start]) {
            System.out.printf("fake: data[%d]=%d < data[%d]=%d\n", start, Data[start], end, Data[end]);
            int tmp = Data[start];
            while (start != end) Data[start++] = tmp;
        } else {
            float gap = (float)(Data[start] - Data[end]) / (end - start);
            while (start != end) {
                float liner = (int)((end - start) * gap + Data[end]);
                if (Math.abs(Data[start] - liner) > 2000) Data[start] = (int)(liner);
                else if (Math.abs(Data[start] - liner) > 1000) Data[start] = (int)(Data[start] * 0.5 + liner * 0.5);
                else Data[start] = (int)(Data[start] * 0.8 + liner * 0.2);
                start++;
            }
        }
    }

    /*
     * return:
     *   -1: no steady found
     * */
    private int nextSteady(int start) {
        if (Data == null) return -1;
        for (; start + SteadyCount <= Data.length; ++start) {
            if (isSteady(start, start + SteadyCount - 1)) return start;
        }
        return -1;
    }
    
    private boolean isSteady(int s, int e) {
        if (Data == null) return false;
        for (; s < e; ++s) {
            if (Math.abs(Data[s] - Data[s + 1]) >= Accept) return false;
        }
        return true;
    }
    
    /*
    * smoothly update noise of Data[up, si)
    * up: unstable position
    * si: stable start index
    * */
    private void smoothUpdate(int up, int si) {
        int gap = (Data[si] - Data[si + SteadyCount - 1]) / SteadyCount;
        while (up != si) {
            Data[up] = (si - up) * gap + Data[si];
            up++;
        }
    }

    /*
    * return:
    *   -1: no noise found
    * other: noise in higher pos
    * */
    private int nextNoise(int start) {
        while (start < Data.length - 1) {
            if (Math.abs(Data[start] - Data[start + 1]) > Threshold) return start + 1;
            start++;
        }
        return -1;
    }
}
